/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.srv;

public class B2102 extends ioPortT {
	static final int    CHANNELS    = 16;
    static final int    CHANNEL0    = 0x0001;
    static final int    CHANNEL1    = 0x0002;
    static final int    CHANNEL2    = 0x0004;
    static final int    CHANNEL3    = 0x0008;
    static final int    CHANNEL4    = 0x0010;
    static final int    CHANNEL5    = 0x0020;
    static final int    CHANNEL6    = 0x0040;
    static final int    CHANNEL7    = 0x0080;
    static final int    CHANNEL8    = 0x0100;
    static final int    CHANNEL9    = 0x0200;
    static final int    CHANNEL10   = 0x0400;
    static final int    CHANNEL11   = 0x0800;
    static final int    CHANNEL12   = 0x1000;
    static final int    CHANNEL13   = 0x2000;
    static final int    CHANNEL14   = 0x4000;
    static final int    CHANNEL15   = 0x8000;


    byte    m_relay1mask;
    byte    m_relay2mask;
    byte    m_state[];


	/**
	 * create an instance of B2102
	 * 
	 * @see ioPortT#ioPortT(String)
	 */
	B2102(String name) {
		super(name);
        m_state = new byte[CHANNELS];

		setBase(0x03E0);
		setDelay(100);
	}


    /**
     * clear states and masks for all channels on B2102
     */
    void clear() {
        for (int i=0; i<CHANNELS; i++)
            m_state[i] = 0;
        m_relay1mask = 0;
        m_relay2mask = 0;
    }


	/**
	 *	reset all channels on B2102 
	 */
    void reset() {
        clear();
		if (m_isOpen) {
	        write(0,m_relay1mask);
    	    write(1,m_relay2mask);
		}
    }


	/**
	 * set the base addr of the B2102
	 * 
	 * @see ioPortT#setBase(int)
	 */
    public int setBase(int b)  {
        int x = super.setBase(b);
        clear();
        return x;
    }


	/**
	 * get the mask for channels 0-7
	 * 
	 * @return byte
	 */
    byte getMask1() {
        return m_relay1mask;
    }


	/**
	 * get the mask for channels 8-15
	 * 
	 * @return byte
	 */
    byte getMask2() {
        return m_relay2mask;
    }


	/**
	 * set the mask for channels 0-7
	 * 
	 * @param m
	 */
    void setMask1(byte m) {
        m_relay1mask = m;
    }


	/**
	 * set the mask for channels 8-15
	 * 
	 * @param m
	 */
    void setMask2(byte m) {
        m_relay2mask = m;
    }


	/**
	 * set the on/off state of a channel
	 * 
	 * @param channel
	 * @param state
	 */
    void setState(int channel, int state) {
		if (m_isOpen == false)
			return;
			
        if (channel < 1 || channel > CHANNELS) {
            jauto.display("invalid channel:"+channel);
            return;
        }

        int base0 = channel - 1;
        switch(channel) {
            case  1:    channel = CHANNEL0;     break;
            case  2:    channel = CHANNEL1;     break;
            case  3:    channel = CHANNEL2;     break;
            case  4:    channel = CHANNEL3;     break;
            case  5:    channel = CHANNEL4;     break;
            case  6:    channel = CHANNEL5;     break;
            case  7:    channel = CHANNEL6;     break;
            case  8:    channel = CHANNEL7;     break;
            case  9:    channel = CHANNEL8;     break;
            case 10:    channel = CHANNEL9;     break;
            case 11:    channel = CHANNEL10;    break;
            case 12:    channel = CHANNEL11;    break;
            case 13:    channel = CHANNEL12;    break;
            case 14:    channel = CHANNEL13;    break;
            case 15:    channel = CHANNEL14;    break;
            case 16:    channel = CHANNEL15;    break;
            default:    jauto.display("invalid channel:"+channel);
                        return;
        }

        m_state[base0] = (byte)state;

        if (state==0)
            off(channel);
        else
            on(channel);
    }
    
    
    /**
     * get the state of a channel
     * 
     * @param channel
     * 
     * @return int
     */
	int getState(int channel) {
		if (m_isOpen == false)
			return -1;
			
		if (channel < 1 || channel > CHANNELS) {
			jauto.display("invalid channel:"+channel);
			return 0;
		}

		int base0 = channel - 1;
		switch(channel) {
			case  1:    channel = CHANNEL0;     break;
			case  2:    channel = CHANNEL1;     break;
			case  3:    channel = CHANNEL2;     break;
			case  4:    channel = CHANNEL3;     break;
			case  5:    channel = CHANNEL4;     break;
			case  6:    channel = CHANNEL5;     break;
			case  7:    channel = CHANNEL6;     break;
			case  8:    channel = CHANNEL7;     break;
			case  9:    channel = CHANNEL8;     break;
			case 10:    channel = CHANNEL9;     break;
			case 11:    channel = CHANNEL10;    break;
			case 12:    channel = CHANNEL11;    break;
			case 13:    channel = CHANNEL12;    break;
			case 14:    channel = CHANNEL13;    break;
			case 15:    channel = CHANNEL14;    break;
			case 16:    channel = CHANNEL15;    break;
			default:	jauto.display("invalid channel:"+channel);
						return 0;
		}

		return (int)m_state[base0];
	}



	/**
	 * turn a channel on
	 * 
	 * @param channel
	 */
	private void on(int channel) {
        if (m_base == 0)
            return;

        if (channel <= CHANNEL7) {
            m_relay1mask |= (byte)channel;
            write(0,m_relay1mask);
        }
        else {
            m_relay2mask |= (byte)(channel>>8);
            write(1,m_relay2mask);
        }
    }


	/**
	 * turn a channel off
	 * 
	 * @param channel
	 */
    private void off(int channel) {
        if (m_base == 0)
            return;

        if (channel <= CHANNEL7) {
            m_relay1mask &= ~((byte)channel);
            write(0,m_relay1mask);
        }
        else {
            m_relay2mask &= ~((byte)(channel>>8));
            write(1,m_relay2mask);
        }
    }


	/**
	 * convert B2102 channel id's into 0 based numbers
	 * 
	 * @param idx
	 * 
	 * @return int
	 */
    int getChannelId(int idx) {
        switch (idx) {
            case 0:     return CHANNEL0;
            case 1:     return CHANNEL1;
            case 2:     return CHANNEL2;
            case 3:     return CHANNEL3;
            case 4:     return CHANNEL4;
            case 5:     return CHANNEL5;
            case 6:     return CHANNEL6;
            case 7:     return CHANNEL7;
            case 8:     return CHANNEL8;
            case 9:     return CHANNEL9;
            case 10:    return CHANNEL10;
            case 11:    return CHANNEL11;
            case 12:    return CHANNEL12;
            case 13:    return CHANNEL13;
            case 14:    return CHANNEL14;
            case 15:    return CHANNEL15;
        }
        return -1;
    }

}
