/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.srv;

import JAuto.util.*;

import java.util.*;
import java.net.*;

public class clientConnectionT extends Thread {
	private static LinkedList	_list = new LinkedList();
	private static int		_id = 0;
	
	int		id;
	srvT		srv;
	sockT 		sock;
	clientT		client;
	
	/**
	 * 
	 * @param srv
	 * @param socket
	 */
	clientConnectionT(srvT srv, Socket socket) {
		this.id = _id++;
		this.srv = srv;
		this.sock = new sockT(socket);
		this.client = null;

		synchronized (_list) {
			_list.add(this);
		}
	
		start();
	}


	/**
	 * 
	 * @param msg
	 * @return boolean
	 */
	private clientT _logon(msgT msg) {
		//
		// verify user name and password
		//
		clientT client = clientT.fnd(msg.data1, msg.data2);
		if (client == null) {
			System.out.println("client: logon error");
			//msg.dump();
			return null;
		}

		//System.out.println("client logon OK:"+msg.data1);
		
		//
		// send current settings to client
		//
		sendControllers();
		sendRelays();
		sendSensors();		
		return client;
	}


	/**
	 * 
	 */
	public void sendControllers() {
		Iterator itr = ioPortT.boards.iterator();
		
		while (itr.hasNext()) {
			ioPortT	io = (ioPortT)itr.next();
			msgT 	msg = new msgT();
			
			msg.cmd = sysMsgT.MSG_EDIT_BOARD;
			msg.board = io.getName();
			msg.int1 = io.getBase();
			msg.int2 = io.getEnabled() ? 1:0; 
			
			sock.sendMsg(msg);
		}
	}

	
	/**
	 * 
	 */
	public void sendRelays() {
		Iterator itr = deviceT.relays.iterator();
		
		while (itr.hasNext()) {
			relayT	relay = (relayT)itr.next();
			msgT 	msg = new msgT();
			
			msg.bldRelayMsg(relay);
			sock.sendMsg(msg);
		}
	}


	/**
	 * 
	 */
	public void sendSensors() {
		Iterator itr = deviceT.sensors.iterator();
		
		while (itr.hasNext()) {
			sensorT	sensor = (sensorT)itr.next();
			msgT 	msg = new msgT();
			
			msg.bldSensorMsg(sensor);			
			sock.sendMsg(msg);
		}
	}


	/**
	 * 
	 */
	public void run() {
		ioPortT	io;
		relayT	relay;
		relayT	newRelay;
		relayT	oldRelay;
		sensorT newSensor;
		sensorT oldSensor;
		msgT    msg = new msgT();

		//
		// loop on reading of socket
		//
		while (true) {
			if (sock.readMsg(msg) < 0) {
				System.out.println("client: sock.readMsg(msg) error");
				break;
			}
			//msg.dump();
			
			if (client == null) {
				//
				// is it a client login?
				//
				if (msg.cmd != sysMsgT.MSG_LOGON) 
					break;
					
				client = _logon(msg);
				if (client == null)
					break;
			}
			
			//
			// process the client msg
			//
			switch (msg.cmd) {
				case sysMsgT.MSG_EDIT_RELAY:
					// the client is changing the settings of a relay
					newRelay = relayT.loadRelay(msg);
					oldRelay = null;
					if (msg.data4.length() > 0) {
						oldRelay = deviceT.fndRelay(msg.data4);
					}
					sysMsgT.postMsg(sysMsgT.MSG_EDIT_RELAY, newRelay, oldRelay);					 
					break;
					
				case sysMsgT.MSG_EDIT_SENSOR:
					//the client is changing the settings of a sensor
					newSensor = sensorT.loadSensor(msg);
					oldSensor = null;
					if (msg.data4.length() > 0) {
						oldSensor = deviceT.fndSensor(msg.data4);
					}
					sysMsgT.postMsg(sysMsgT.MSG_EDIT_SENSOR, newSensor, oldSensor);					 
					break;
					
				case sysMsgT.MSG_DEL_RELAY:
					sysMsgT.postMsg(sysMsgT.MSG_DEL_RELAY, msg.device);
					break;					
					
				case sysMsgT.MSG_DEL_SENSOR:
					sysMsgT.postMsg(sysMsgT.MSG_DEL_SENSOR, msg.device);
					break;
					
				case sysMsgT.MSG_EDIT_BOARD:
					io = ioPortT.fnd(msg.board);
					if (io != null) {
						sysMsgT.postMsg(sysMsgT.MSG_EDIT_BOARD, io, msg.int1, msg.int2);
					}
					break;

				case sysMsgT.MSG_OVERRIDE:
					relay = deviceT.fndRelay(msg.device);
					if (relay != null)  {
						sysMsgT.postMsg(sysMsgT.MSG_OVERRIDE, relay, msg.int1);
					}
					break;
			}

		}
		
		synchronized (_list) {
			_list.remove(this);
		}

	}


	/**
	 * 
	 * @param id
	 * @return clientConnectionT
	 */
	public static clientConnectionT fnd(int id) {
		synchronized (_list) {
			Iterator itr = _list.iterator();
			
			while (itr.hasNext()) {
				clientConnectionT conn = (clientConnectionT)itr.next();
				if (id == conn.id) {
					return conn;
				}
			}
		}
		return null;
	}


	/**
	 * 
	 * @param sensor
	 */
	public static void procSensor(sensorT sensor) {
		msgT    msg = new msgT();
		
		msg.cmd = sysMsgT.MSG_PROC_SENSOR;
		msg.device = sensor.getName();
		msg.int1 = sensor.getSensorDat();

		_broadcast(msg);
	}
	
	
	/**
	 * 
	 * @param relay
	 */
	public static void procRelay(relayT relay) {
		msgT    msg = new msgT();
		
		msg.cmd = sysMsgT.MSG_PROC_RELAY;
		msg.device = relay.getName();
		msg.state = relay.getState() ? 1:0;

		_broadcast(msg);
	}
		
	
	/**
	 * 
	 * @param relay
	 */
	public static void overrideRelay(relayT relay, int state) {
		msgT    msg = new msgT();
		
		msg.cmd = sysMsgT.MSG_OVERRIDE;
		msg.device = relay.getName();
		msg.state = relay.getState() ? 1:0;
		msg.int1 = state;
		
		_broadcast(msg);
	}

	/**
	 * 
	 * @param relay
	 */
	public static void editBoard(ioPortT io) {
		msgT	msg = new msgT();
		
		msg.cmd = sysMsgT.MSG_EDIT_BOARD;
		msg.board = io.getName();
		msg.int1 = io.getBase();
		msg.int2 = io.getEnabled() ? 1:0;
		
		_broadcast(msg);
	}

	/**
	 * 
	 * @param relay
	 */
	public static void editRelay(relayT oldRelay, relayT newRelay) {
		msgT    msg = new msgT();
		
		msg.bldRelayMsg(newRelay);		
		if (oldRelay != null)
			msg.data4 = oldRelay.getName();
		
		_broadcast(msg);
	}



	/**
	 * 
	 * @param newSensor
	 * @param oldSensor
	 */
	public static void editSensor(sensorT newSensor, sensorT oldSensor) {
		msgT    msg = new msgT();
		
		msg.bldSensorMsg(newSensor);		
		if (oldSensor != null)
			msg.data4 = oldSensor.getName();
		
		_broadcast(msg);
	}
	
	/**
	 * 
	 * @param relay
	 */
	public static void delRelay(String name) {
		msgT    msg = new msgT(sysMsgT.MSG_DEL_RELAY, name);
		
		_broadcast(msg);
	}
	

	/**
	 * 
	 * @param name
	 */
	public static void delSensor(String name) {
		msgT    msg = new msgT(sysMsgT.MSG_DEL_SENSOR, name);
		
		_broadcast(msg);
	}
	

	private static void _broadcast(msgT msg) {
		synchronized (_list) {
			Iterator itr = _list.iterator();			
			while (itr.hasNext()) {
				clientConnectionT conn = (clientConnectionT)itr.next();
				conn.sock.sendMsg(msg);
			}
		}
	}

	
}
