/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.srv;

public class condT {
	public	static	final	int	EQ	= 1;
	public	static	final	int	NE	= 2;
	public	static	final	int	LT	= 3;
	public	static	final	int	LE	= 4;
	public	static	final	int	GT	= 5;
	public	static	final	int	GE	= 6;
	
	String		device;
	int		op;
	int		val;	
	int		tblRow;

	/**
	 * Copy constructor.
	 * 
	 * @param cond
	 */
	public condT(condT cond) {
		this.device = new String(cond.device);
		this.op = cond.op;
		this.val = cond.val;
	}

	
	/**
	 * create an instance of a condition
	 * 
	 * @param device
	 * @param op
	 * @param val
	 */
	public condT(String device, String op, String val) {
		setDevice(device);
		
		setOp(op);
		setVal(val);
	}
	
	
	/**
	 * set the device this condition refers to
	 * 
	 * @param device
	 */
	public void setDevice(String device) {
		this.device = device;
	}
	
	
	/**
	 * set the operation this condition uses
	 * 
	 * @param op
	 */
	public void setOp(String op) {
		if ("=".equalsIgnoreCase(op)) {
			this.op = EQ; 
		}
		else if ("==".equalsIgnoreCase(op)) {
			this.op = EQ;
		}
		else if ("!=".equalsIgnoreCase(op)) {
			this.op = NE;
		}
		else if ("<".equalsIgnoreCase(op)) {
			this.op = LT;
		}
		else if ("<=".equalsIgnoreCase(op)) {
			this.op = LE;
		}
		else if (">".equalsIgnoreCase(op)) {
			this.op = GE;
		}
		else if (">=".equalsIgnoreCase(op)) {
			this.op = GE;
		}
		else this.op = EQ;
	}


	/**
	 * set the value this condition uses
	 * 
	 * @param val
	 */	
	public void setVal(String val) {
		try { this.val = Integer.parseInt(val); }
		catch (Exception e) {
			if (val.equalsIgnoreCase("ON"))  
				this.val = 1; 
			else
				this.val = 0;
		}
	}
	
	
	/**
	 * check the condition for true/false
	 * 
	 * @return boolean
	 */
	public boolean chk() {
		deviceT dev = deviceT.fnd(device);
		if (dev == null)
			return false;

		if (dev instanceof sensorT) {
			int tval = ((sensorT)dev).getSensorDat();			
			switch (op) {
				case EQ:	return (tval == val);
				case NE:	return	(tval != val);
				case LT:	return	(tval < val);
				case LE:	return	(tval <= val);
				case GT:	return	(tval > val);
				case GE:	return	(tval >= val);
			}
			return false;			
		}
		
		if (dev instanceof relayT) {
			boolean tval = (val == 1) ? true:false;
			if (op == EQ) 	return dev.getState() == tval;
			if (op == NE) 	return dev.getState() != tval;
		}				
		
		return false;
	}


	/**
	 * Returns the device.
	 * 
	 * @return String
	 */
	public String getDevice() {
		return device;
	}
	

	/**
	 * Returns the operation.
	 * 
	 * @return int
	 */
	public int getOp() {
		return op;
	}
	

	/**
	 * get the operation in string form
	 * 
	 * @return String
	 */
	public String getOpString() {
		switch (op) {
			case EQ:	return "==";
			case NE:	return	"!=";
			case LT:	return	"<";
			case LE:	return	"<=";
			case GT:	return	">";
			case GE:	return	">=";
		}
		return "?";
	}


	/**
	 * Return the val used in this condition.
	 * 
	 * @return int
	 */
	public int getVal() {
		return val;
	}
	

	/**
	 * Returns the tblRow.
	 * 
	 * @return int
	 */
	public int getTblRow() {
		return tblRow;
	}
	

	/**
	 * Sets the tblRow.
	 * 
	 * @param tblRow The tblRow to set
	 */
	public void setTblRow(int tblRow) {
		this.tblRow = tblRow;
	}

}

