/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.srv;

import java.util.*;
import javax.swing.*;

public class deviceT {
	public	static	final	int	ON    = 1;
	public	static	final	int	OFF   = 0;

	public	static	final	int	TYP_INPUT = 1;
	public	static	final	int	TYP_RELAY = 2;
	
	public	static	LinkedList	relays = new LinkedList();
	public	static	LinkedList	sensors = new LinkedList();
	
	protected	srvT		srv = srvT.getInstance();
	
	protected	String		name;
	protected	boolean	state;
	protected	boolean	enable;
	protected	int		typ;
	//protected	int		boardTyp;
	protected	ioPortT		board;
	protected	int		channel;
	
	protected	String		slaveOfName;	//used by SLAVE type relays
	protected	relayT		slaveOf;		//used by SLAVE type relays
	protected	int		onSecs;			//used by CYCLE type relays
	protected	int		offSecs;		//used by CYCLE type relays
	protected	Date		onTime;			//used by CLOCK type relays
	protected	Calendar	onCalendar;		//used by CLOCK type relays
	protected	Date		offTime;		//used by CLOCK type relays
	protected	Calendar	offCalendar;	//used by CLOCK type relays

	private	int		tblRow;

	
	
	/**
	 * create a deviceT instance  
	 * 
	 * @param name	the name of the device
	 * @param typ	the type of device (TYP_INPUT or TYP_RELAY)
	 */
	deviceT(String name, int typ) {
		this.name = new String(name);
		this.typ = typ;
		this.state = false;
		this.enable = true;
		
		this.onCalendar = null;
		this.offCalendar = null;
		this.slaveOf = null;
	}

	/**
	 * copy constructor
	 * 
	 * @param dev
	 */
	public deviceT(deviceT dev) {
		this.name        = dev.name;
		this.state       = dev.state;
		this.enable      = dev.enable;
		this.typ         = dev.typ;
		//this.boardTyp    = dev.boardTyp;
		this.board       = dev.board;
		this.channel     = dev.channel;

		this.onTime      = dev.onTime;
		this.onCalendar  = dev.onCalendar;
		this.offTime     = dev.offTime;
		this.offCalendar = dev.offCalendar;
		this.slaveOf = dev.slaveOf;		
	}
	
	public ioPortT getBoard()	{return board;}
	//public int getBoardTyp()	{return boardTyp;}
	public int getChannel()	{return channel;}
	public boolean isEnable()	{return enable;}
	public String getName()	{return name;}
	public boolean getState()	{return state;}
	public int getDeviceTyp()	{return typ;}


	/**
	 * Sets the board type.
	 * 
	 * @param board The board to set
	 */
	public void setBoard(ioPortT board) {
		this.board = board;
	}


	/**
	 * Sets the board type.
	 * 
	 * @param board The board to set
	 */
	public void setBoard(String board) {
		this.board = ioPortT.fnd(board);
	}


	/**
	 * get the board name
	 * 
	 * @return String
	 */
	public String getBoardName() {
		if (board instanceof B2102) 
			return "B2102";
		if (board instanceof B3001) 
			return "B3001";
		if (board instanceof pportT)
			return "Parallel";
			
		return "?????";
	}


	/**
	 * this device is a slave; set it's slaveOf fld
	 * 
	 * @param relay
	 */
	public void setSlaveDeviceName(String name) {
		slaveOfName = name;
	}
	
	/**
	 * Returns the slaveOfName.
	 * @return String
	 */
	public String getSlaveDeviceName() {
		return slaveOfName;
	}


	/**
	 * this device is a slave; set it's slaveOf fld
	 * 
	 * @param relay
	 */
	public void setSlaveDevice(relayT relay) {
		slaveOf = relay;
	}

	/**
	 * this device is a slave; set it's slaveOf fld
	 * 
	 * @param relay
	 */
	public void setSlaveDevice(String relay) {
		slaveOf = deviceT.fndRelay(relay);
	}
	

	/**
	 * get the device this device is a slave of
	 * 
	 * @return relayT
	 */
	public relayT getSlaveDevice() {
		return slaveOf;
	}


	/**
	 * Sets the channel.
	 * 
	 * @param channel The channel to set
	 */
	public void setChannel(int channel) {
		this.channel = channel;
	}


	/**
	 * Sets the enable state.
	 * 
	 * @param enable The enable to set
	 */
	public void setEnable(boolean enable) {
		this.enable = enable;
	}


	/**
	 * Sets the device name.
	 * 
	 * @param name The name to set
	 */
	public void setName(String name) {
		this.name = name;
	}


	/**
	 * Sets the device typ.
	 * 
	 * @param typ The typ to set
	 */
	public void setDeviceTyp(int typ) {
		this.typ = typ;
	}


	/**
	 * Set the device state.
	 * 
	 * @param flg
	 */
	public void setState(boolean flg) {
		this.state = flg;
	}


	/**
	 * Toggle the device state.
	 */
	public void toggleState() {
		if (this.state == true)
			setState(false);
		else
			setState(true);
	}


	/**
	 * set the on/off time flds of this device
	 * 
	 * @param onOff
	 * @param h
	 * @param m
	 * @param s
	 */	
	public void setTime(int onOff, int h, int m, int s) {
		if (onOff == deviceT.OFF) {
			offCalendar = Calendar.getInstance();
			offCalendar.clear();
			offCalendar.set(Calendar.DAY_OF_MONTH, 1);
			offCalendar.set(Calendar.HOUR_OF_DAY, h);
			offCalendar.set(Calendar.MINUTE, m);
			offCalendar.set(Calendar.SECOND, s);
			this.offTime = offCalendar.getTime();
		}
		else {
			onCalendar = Calendar.getInstance();
			onCalendar.clear();
			onCalendar.set(Calendar.DAY_OF_MONTH, 1);
			onCalendar.set(Calendar.HOUR_OF_DAY, h);
			onCalendar.set(Calendar.MINUTE, m);
			onCalendar.set(Calendar.SECOND, s);
			this.onTime = onCalendar.getTime();
		}
	}


	/**
	 * Determine if the device should be on or off at startup.
	 * 
	 * @return boolean
	 */
	public boolean chkOnOffTime() {
		boolean	debug = false;
		
		if (onCalendar==null || offCalendar==null)
			return true;

		Date 		timeOn = onTime;		//onCalendar.getTime();
		Date 		timeOff = offTime;		//onCalendar.getTime();			
		Calendar	now = Calendar.getInstance();
		
		now.set(Calendar.YEAR, onCalendar.get(Calendar.YEAR));
		now.set(Calendar.MONTH, onCalendar.get(Calendar.MONTH));
		now.set(Calendar.DAY_OF_MONTH, onCalendar.get(Calendar.DAY_OF_MONTH));

		if (debug) {
			srv.display("1 timeOn:"+timeOn);
			srv.display("1 timeOff:"+timeOff);
			srv.display("1 now:"+now.getTime()+"\n");
		}
		
		if (timeOff.compareTo(timeOn) == -1) {
			//
			// off time is < on time; adjust to handle midnight roll over
			//
			Calendar off = Calendar.getInstance();
			off.setTime(offCalendar.getTime());
			off.add(Calendar.DATE,1);
			timeOff = off.getTime();
			
			if (debug) {
				srv.display("2 timeOn:"+timeOn);
				srv.display("2 timeOff:"+timeOff);
				srv.display("2 now:"+now.getTime()+"\n");
			}
		
			if (timeOn.compareTo(now.getTime()) == 1) {
				//
				// on time > cur time; adjust cur time
				//
				now.add(Calendar.DATE,1);
			}
		}
		
		if (debug) {
			srv.display("3 timeOn:"+timeOn);
			srv.display("3 timeOff:"+timeOff);
			srv.display("3 now:"+now.getTime());
		}

		
		Date nowTime = now.getTime();		
		if ((nowTime.compareTo(timeOn) >= 0) && (nowTime.compareTo(timeOff) <= 0)) {
			return true;
		}
		return false;
	}


	/**
	 * virtual function for children
	 */
	public void process() {
	}


	/**
	 * Returns the offSecs.
	 * @return int
	 */
	public int getOffSecs() {
		return offSecs;
	}


	/**
	 * Returns the onSecs.
	 * @return int
	 */
	public int getOnSecs() {
		return onSecs;
	}


	/**
	 * Returns the offTime.
	 * @return Date
	 */
	public Date getOffTime() {
		return offTime;
	}


	/**
	 * Returns the onTime.
	 * @return Date
	 */
	public Date getOnTime() {
		return onTime;
	}


	/**
	 * Returns the offCalendar.
	 * @return Calendar
	 */
	public Calendar getOffCalendar() {
		return offCalendar;
	}


	/**
	 * Returns the onCalendar.
	 * @return Calendar
	 */
	public Calendar getOnCalendar() {
		return onCalendar;
	}


	/**
	 * Returns the tblRow.
	 * @return int
	 */
	public int getTblRow() {
		return tblRow;
	}


	/**
	 * Sets the tblRow.
	 * @param tblRow The tblRow to set
	 */
	public void setTblRow(int tblRow) {
		this.tblRow = tblRow;
	}



	/**
	 * 
	 * @param dev
	 */
	public static void addToList(deviceT dev) {
		if (dev.typ == TYP_RELAY)
			relays.add(dev);
		else
			sensors.add(dev);			
	}
	

	/**
	 * find a device by name
	 * 
	 * @param name
	 * 
	 * @return deviceT
	 */
	public static deviceT fnd(String name) {
		deviceT	dev = fndSensor(name);
		if (dev != null)
			return dev;
		return fndRelay(name);			
	}
	

	/**
	 * find a relay by name
	 * 
	 * @param name
	 * 
	 * @return deviceT
	 */
	public static sensorT fndSensor(String name) {
		synchronized (sensors) {
			Iterator itr = sensors.iterator();
			while (itr.hasNext()) {
				sensorT dev = (sensorT)itr.next();
				if (name.equals(dev.getName()))
					return dev;
			}
		}		
		return null;
	}
	

	/**
	 * find a relay by name
	 * 
	 * @param name
	 * 
	 * @return deviceT
	 */
	public static relayT fndRelay(String name) {
		synchronized (relays) {	
			Iterator itr = relays.iterator();
			while (itr.hasNext()) {
				relayT dev = (relayT)itr.next();
				if (name.equals(dev.getName()))
					return dev;
			}
		}		
		return null;			
	}

	
	/**
	 * delete a device
	 * 
	 * @param dev
	 */
	public static void del(deviceT dev) {
		if (dev != null) {
			sensors.remove(dev);
			relays.remove(dev);
		}
	}

	/**
	 * delete a device
	 * 
	 * @param dev
	 */
	public static void del(String name) {
		deviceT	dev = fndSensor(name);
		
		if (dev != null) {
			sensors.remove(dev);
			return;
		}
			
		dev = fndRelay(name);
		if (dev != null) {
			relays.remove(dev);
		}
	}


	/**
	 * load all relay and sensor names into a combo box
	 * 
	 * @param cb
	 */
	public static void loadNamesComboBox(JComboBox cb) {
		loadRelaysComboBox(cb);
		loadSensorsComboBox(cb);
	}		
		
	
	/**
	 * load all relay names into a combo box
	 * 
	 * @param cb
	 */
	public static void loadRelaysComboBox(JComboBox cb) {
		synchronized (deviceT.class) {
			Iterator itr = deviceT.relays.iterator();
			while (itr.hasNext()) {
				deviceT dev = (deviceT)itr.next();
				cb.addItem(dev.getName());
			}
		}
	}		
		
	
	/**
	 * load all sensor names into a combo box
	 * 
	 * @param cb
	 */
	public static void loadSensorsComboBox(JComboBox cb) {
		synchronized (deviceT.class) {
			Iterator itr = deviceT.sensors.iterator();
			while (itr.hasNext()) {
				deviceT dev = (deviceT)itr.next();
				cb.addItem(dev.getName());
			}
		}
	}
	
			
}
