/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.srv;

import java.util.*;
import java.io.*;
import javax.swing.*;

public class iniT {
	srvT		srv;
	Properties 	prop = new Properties();
	
	public	String		sqlHost;
	public	String		sqlDB;
	public	String		sqlUser;
	public	String		sqlPW;
	public	boolean	sqlDetail;


	/**
	 * Constructor for iniT.
	 */
	public iniT() {
		srv = srvT.getInstance();
	}

	/**
	 * load sql settings
	 */
	public void loadSqlSettings() {
		try {
			prop.load(new FileInputStream(srv.iniName));
		}
		catch (Exception excp) {
			JOptionPane.showMessageDialog(null,"Error loading "+srv.iniName);
			return;
		}

		sqlHost	= prop.getProperty("sql.host",	"localhost");
		sqlDB	= prop.getProperty("sql.db",	"grow");
		sqlUser = prop.getProperty("sql.user",	"userLogin");
		sqlPW	= prop.getProperty("sql.pw",	"userPW");
		
		String detail = prop.getProperty("sql.detail","no");
		if (detail.equals("no")) 
			sqlDetail = false;
		else
			sqlDetail = true;		
	}
	
	/**
	 * load properties
	 */
	public void loadProp() {
		String	pfx;
		int	seq;

		//
		// load clients
		//
		seq = 0;
		while (true) {
			pfx = "client"+seq;
			String	name = prop.getProperty(pfx+".name", "");
			if (name.length() <= 0)
				break;

			String	pw = prop.getProperty(pfx+".pw", "");
			if (pw.length() <= 0)
				break;

			srv.ui.updClients(new clientT(name,pw));			
			seq++;
		}		

		
		
		//
		// load controller settings
		//
		seq = 0;
		while (true) {
			pfx = "controller"+seq;
			String	board = prop.getProperty(pfx+".board", "");
			if (board.length() <= 0)
				break;

			ioPortT io = ioPortT.fnd(board);
			if (io == null) 
				continue;
				
			String	addr = prop.getProperty(pfx+".addr", "0");
			int i = Integer.parseInt(addr,16);
			io.setBase(i);
						
			String	enable = prop.getProperty(pfx+".enable", "NO");
			if (enable.equalsIgnoreCase("YES"))
				io.setEnabled(true);
			else
				io.setEnabled(false);


			seq++;
		}		

		//
		// load sensors
		//
		seq = 0;
		while (true) {
			pfx = "sensor"+seq;
			String	name = prop.getProperty(pfx+".name", "");
			if (name.length() <= 0)
				break;

			String	board = prop.getProperty(pfx+".board", "");
			if (!board.equalsIgnoreCase("B3001"))
				break;

			String	channel	= prop.getProperty(pfx+".channel", "1");
			
			sensorT sensor = new sensorT(name,srv.b3001,Integer.parseInt(channel));
			
			String	multiplier = prop.getProperty(pfx+".multiplier", "1.0");
			sensor.setMultiplier(Double.parseDouble(multiplier));
			
			deviceT.addToList(sensor);
			srv.ui.updSensor(sensor);
						
			seq++;
		}		

		//
		// load relays
		//
		seq = 0;
		while (true) {
			pfx = "relay"+seq;
			String	name = prop.getProperty(pfx+".name", "");
			if (name.length() <= 0)
				break;

			int	relayTyp;
			String	type = prop.getProperty(pfx+".type", "");
			if (type.equalsIgnoreCase("CLOCK"))
				relayTyp = relayT.CLOCK;
			else if (type.equalsIgnoreCase("CYCLE"))
				relayTyp = relayT.CYCLE;
			else if (type.equalsIgnoreCase("SLAVE"))
				relayTyp = relayT.SLAVE;
			else if (type.equalsIgnoreCase("COND"))
				relayTyp = relayT.COND;
			else
				break;

			ioPortT	io;
			String	board = prop.getProperty(pfx+".board", "");
			if (board.equalsIgnoreCase("B2102"))
				io = srv.b2102;
			else if (board.equalsIgnoreCase("Parallel"))
				io = srv.pport;
			else
				break;


			String	channel	= prop.getProperty(pfx+".channel", "");
			if (channel.length() <= 0)
				break;

			relayT	relay = new relayT(name,relayTyp,io,Integer.parseInt(channel));
			
			if (relayTyp == relayT.SLAVE) {
				String	txt = prop.getProperty(pfx+".time0", "");
				if (txt.length() > 0) {
					relay.setSlaveDeviceName(txt);
					//relay.setSlaveDevice(deviceT.fndRelay(txt));
				}
			}
			else {
				loadRelayTimes(pfx,relay);
			}
			loadRelayConditions(pfx,relay);

			deviceT.addToList(relay);			
			seq++;
		}
		

		//
		// setup mappings to any slave devices
		//		
		seq = 0;		
		Iterator itr = deviceT.relays.iterator();
		while (itr.hasNext()) {
			relayT relay = (relayT)itr.next();

			if (relay.getRelayTyp() == relayT.SLAVE) {
				relay.setSlaveDevice(relay.getSlaveDeviceName());
			}
			srv.ui.updRelay(relay);
		}
	}
	
	
	/**
	 * load the on/off times of a relay
	 * 
	 * @param relay
	 */
	private void loadRelayTimes(String prefix, relayT relay) {
		int	seq = 0;
		
		while (true) {
			String	key = prefix+".time"+seq;						
			String	txt = prop.getProperty(key, "");
			if (txt.length() <= 0)
				break;
				
			StringTokenizer	st = new StringTokenizer(txt,";");
			if (st.hasMoreTokens()) {
				String onOff = st.nextToken();
				String hours = st.nextToken();
				String mins = st.nextToken();
				String secs = st.nextToken(); 
			
				relay.setTime(Integer.parseInt(onOff),	
							  Integer.parseInt(hours), 
							  Integer.parseInt(mins),
							  Integer.parseInt(secs));
			}
			seq++;
		}
	}
	
	
	/**
	 * load the conditions for a relay
	 * 
	 * @param relay
	 */
	private void loadRelayConditions(String prefix, relayT relay) {
		int	seq = 0;

		while (true) {
			String	key = prefix+".onCond"+seq;						
			String	txt = prop.getProperty(key, "");
			if (txt.length() <= 0)
				break;
				
			StringTokenizer	st = new StringTokenizer(txt,";");
			if (st.hasMoreTokens()) {
				String device	= st.nextToken();
				String operator	= st.nextToken();
				String val		= st.nextToken();
			
				relay.addOnCondition( new condT(device, operator, val) );
			}
			seq++;
		}

		seq = 0;
		while (true) {
			String	key = prefix+".offCond"+seq;						
			String	txt = prop.getProperty(key, "");
			if (txt.length() <= 0)
				break;
				
			StringTokenizer	st = new StringTokenizer(txt,";");
			if (st.hasMoreTokens()) {
				String device	= st.nextToken();
				String operator	= st.nextToken();
				String val		= st.nextToken();
			
				relay.addOffCondition( new condT(device, operator, val) );
			}
			seq++;
		}
	}
	

	/**
	 * save the properties
	 */
	public void saveProp() {
		prop.clear();
		
		String	pfx;
		int	seq;
		Iterator itr;

		//
		// save sql setings 
		//
		prop.setProperty("sql.host",sqlHost);
		prop.setProperty("sql.db", 	sqlDB);
		prop.setProperty("sql.user",sqlUser);
		prop.setProperty("sql.pw", 	sqlPW);
		if (sqlDetail)
			prop.setProperty("sql.detail","yes");
		else
			prop.setProperty("sql.detail","no");

		//
		// save clients
		//
		seq = 0;
		itr = clientT.getList().iterator();
		while (itr.hasNext()) {
			pfx = "client"+seq;
				
			clientT client = (clientT)itr.next();
			prop.setProperty(pfx+".name", client.getName());
			prop.setProperty(pfx+".pw", client.getPW());

			seq++;
		}						
		

		//
		// save controller settings
		//
		seq = 0;
		itr = ioPortT.boards.iterator();
		while (itr.hasNext()) {
			ioPortT io = (ioPortT)itr.next(); 
			
			pfx = "controller"+seq;
			
			prop.setProperty(pfx+".board", io.getName());
			prop.setProperty(pfx+".addr", Integer.toHexString(io.getBase()));
			prop.setProperty(pfx+".enable", io.getEnabled() ? "YES":"NO");

			seq++;
		}		

				
		//
		// save sensors
		//
		seq = 0;		
		itr = deviceT.sensors.iterator();
		while (itr.hasNext()) {
			sensorT sensor = (sensorT)itr.next();

			pfx = "sensor"+seq;
			
			prop.setProperty(pfx+".name", sensor.getName());
			prop.setProperty(pfx+".board", sensor.getBoardName());
			prop.setProperty(pfx+".channel", sensor.getChannel()+"");
			prop.setProperty(pfx+".multiplier", sensor.getMultiplier()+"");

			seq++;
		}

		//
		// save relays
		//
		seq = 0;		
		itr = deviceT.relays.iterator();
		while (itr.hasNext()) {
			relayT relay = (relayT)itr.next();

			pfx = "relay"+seq;
			
			prop.setProperty(pfx+".name", relay.getName());
			prop.setProperty(pfx+".type", relay.getRelayTypStr());
			prop.setProperty(pfx+".board", relay.getBoardName());
			prop.setProperty(pfx+".channel", relay.getChannel()+"");
			
			
			switch (relay.getRelayTyp()) {
				case relayT.CYCLE:
				case relayT.CLOCK:
					saveRelayTimes(pfx,relay);
					saveRelayConditions(pfx,relay);
					break;
				
				case relayT.SLAVE:
					prop.setProperty(pfx+".time0", relay.getSlaveDevice().getName());
					break;
					
				case relayT.COND:
					break;
			}
			saveRelayConditions(pfx,relay);
			
			seq++;
		}

		try {
			prop.store(new FileOutputStream(srv.iniName),null);
		}
		catch (Exception excp) {}
	}


	/**
	 * save the on/off times of a relay
	 * 
	 * @param prefix
	 * @param relay
	 */
	private void saveRelayTimes(String prefix, relayT relay) {
		String	key = prefix+".time0";			
		Calendar cal = relay.getOnCalendar();						
		prop.setProperty(key, "1;"+	
						cal.get(Calendar.HOUR_OF_DAY) + ";" +
						cal.get(Calendar.MINUTE) + ";" +
						cal.get(Calendar.SECOND));

		key = prefix+".time1";
		cal = relay.getOffCalendar();						
		prop.setProperty(key, "0;"+	
						cal.get(Calendar.HOUR_OF_DAY) + ";" +
						cal.get(Calendar.MINUTE) + ";" +
						cal.get(Calendar.SECOND));

	}
	
	
	/**
	 * save the conditions for a relay
	 * 
	 * @param prefix
	 * @param relay
	 */
	private void saveRelayConditions(String prefix, relayT relay) {
		int		seq = 0;
		Iterator	itr = relay.getOnConditions().iterator();
		
		while (itr.hasNext()) {
			String	key = prefix+".onCond"+seq;
			condT	cond = (condT)itr.next();
			
			prop.setProperty(key, 
								cond.getDevice()   +";"+
								cond.getOpString() +";"+
								cond.getVal());	
			seq++;
		}

		seq = 0;
		itr = relay.getOffConditions().iterator();		
		while (itr.hasNext()) {
			String	key = prefix+".offCond"+seq;
			condT	cond = (condT)itr.next();
			
			prop.setProperty(key,
								cond.getDevice()   +";"+
								cond.getOpString() +";"+
								cond.getVal());	
			seq++;
		}
	}
}
