/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.srv;

import java.util.*;
//import javax.swing.*;

public class ioPortT {
	public	static	boolean	jniLoaded = false;
	public	static	LinkedList	boards = new LinkedList();
	
	srvT		jauto = srvT.getInstance();

    String		m_name;
    int		m_base;
    long		m_ioDelay;
	boolean	m_isOpen;
	boolean	m_enabled;
	int		m_tblRow;
	String		m_baseHex;

    /**
     * JNI methods
     */
    public native int	ioOpen();
	public native int	ioClose();
    public native int	ioReadPort(int addr);
    public native int	ioReadPort(int addr, long usec);
	public native int	ioReadB3001(int addr, int channel0);
    public native int	ioWritePort(int addr, byte b);
    public native int	ioWritePort(int addr, byte b, long usec);
    static {
		try {
        	System.loadLibrary("ioPortT");
			jniLoaded = true;
		}
		catch (UnsatisfiedLinkError excp) {
			jniLoaded = false;
			//System.out.println("Error loading 'ioPortT' library");
			//System.out.println("java.library.path = " + System.getProperty("java.library.path")+"\n");
			//JOptionPane.showMessageDialog(null,"Error loading 'ioPortT' library");
			//System.exit(-1);
		}
    }
	/*
	// I wish java had #defines..... 
	public int	ioOpen()									{ return 0; }
	public int	ioClose()									{ return 0; }
    public int	ioReadPort(int addr)						{ return 0; }
    public int	ioReadPort(int addr, long usec)			{ return 0; }
	public int	ioReadB3001(int addr, int channel0)		{ return 0; }
    public int	ioWritePort(int addr, byte b)				{ return 0; }
    public int	ioWritePort(int addr, byte b, long usec)	{ return 0; }
    */


	/**
	 * 
	 */
	public ioPortT() {
		m_name		= "";
		m_base		= 0;
		m_ioDelay	= 0;
		m_isOpen	= false;
		m_enabled	= true;
		m_tblRow	= 0;
		m_baseHex	= "";
		
		jauto = srvT.getInstance();
	}
	

    /**
     * create an instance of ioPortT
     * 
     * @param name
     */
	public ioPortT(String name) {
        m_name		= new String(name);
        m_base		= 0;
        m_ioDelay	= 0;
    	m_isOpen	= false;
		m_enabled	= true;
		m_tblRow	= 0;
		m_baseHex	= "";
		
		boards.add(this);
		jauto = srvT.getInstance();
    }
    
    
    /**
     * get the table row assigned to this entry
     * 
     * @return int
     */
	public int getTblRow() {
		return m_tblRow;
	}
	

	/**
	 * set the table row assigned to this entry
	 * 
	 * @param row
	 */	
	public void setTblRow(int row) {
		m_tblRow = row;
	}


    /**
     * set the base addr of this device
     * 
     * @param b
     * 
     * @return int
     */
	public int setBase(int b)  {
        int oldBase = m_base;
        m_base = b;
		m_baseHex = "0x" + Integer.toHexString(m_base).toUpperCase();
        return oldBase;
    }


	/**
	 * get the base addr of this device
	 * 
	 * @return int
	 */
	public int getBase()  {
		return m_base;
	}
	
	
	/**
	 * get the base addr of this device as a hex string
	 * 
	 * @return String
	 */
	public String getBaseHexStr()  {
		return m_baseHex;
	}


	/**
	 * set the access delay used by this device (micro sec's)
	 * 
	 * @param usec
	 */
	public void setDelay(long usec) {
    	m_ioDelay = usec;
    }


	/**
	 * is this device enabled?
	 * 
	 * @return boolean
	 */
	public boolean getEnabled() {
		return m_enabled;
	}

	
	/**
	 * set enable state of this device
	 * 
	 * @param state
	 * 
	 * @return boolean
	 */
	public boolean setEnabled(boolean state) {
		boolean s = m_enabled;
		m_enabled = state;
		return s;
	}
	
	/**
	 * 
	 * @return String
	 */
	public String getName() {
		return m_name;
	}

	/**
	 * 
	 * @param name
	 */
	public void setName(String name) {
		m_name = name;
	}


    /**
     * open the port
     *
     * @return int
     */
    int open() {
		if (!jniLoaded)
			return 0;
		
        if (m_isOpen)
            return -1;

        if (m_base == 0) {
            jauto.display("ioPortT::open->invalid base address");
            return -1;
        }

        if (ioOpen() == -1) {
            jauto.display("ioPortT::open->JNI ioOpen error");
            return -1;
        }

        m_isOpen = true;
        return 0;
    }


	/**
	 * close the port
	 */
    void close() {
		if (!jniLoaded)
			return;
			
    	if (m_isOpen) {
        	if (ioClose() != 0) {
            	jauto.display("ioPortT::close->JNI ioClose error");
        	}
    		m_isOpen = false;
    	}
    }


	/**
	 * read a byte from the port
	 *
	 * @param offset
	 *
	 * @return int
	 */
    int read(int offset) {
		if (!jniLoaded)
			return 0;
		if (m_isOpen==false)
			return -1;
        if (m_base == 0) {
            jauto.display("ioPortT::read->invalid base address");
            return -1;
        }
        return ioReadPort(m_base+offset);
		//return ioReadPort(m_base+offset, m_ioDelay);
    }


	/**
	 * write a byte to the port
	 *
	 * @param offset
	 * @param b
	 *
	 * @return int
	 */
    int write(int offset, byte b) {
		if (!jniLoaded)
			return 0;
		if (m_isOpen==false)
			return -1;
        if (m_base == 0) {
            jauto.display("ioPortT::write->invalid base address");
            return -1;
        }
        return ioWritePort(m_base+offset, b);
		//return ioWritePort(m_base+offset, b, m_ioDelay);
    }


	/**
	 * find a device by name
	 * 
	 * @param name
	 * 
	 * @return ioPortT
	 */    
	public static ioPortT fnd(String name) {
		ioPortT		io;
		Iterator	itr;
		
		synchronized (boards) {
			itr = boards.iterator();
			while (itr.hasNext()) {
				io = (ioPortT)itr.next();
				if (name.equalsIgnoreCase(io.m_name))
					return io;
			}
		}
		return null;			
	}
}
