/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.srv;

import JAuto.util.*;
import java.net.*;


/**
 * System message thread class
 */
public class msgThreadT extends Thread {
    private srvT     m_srv   = null;

    /**
     * constructor
     */
    msgThreadT(srvT srv) {
        m_srv = srv;
    }

	/**
	 * Execution of the thread is here.
	 */
	public void run() {
		if (m_srv.appType == srvT.SERVER)
			runServer();
		else if (m_srv.appType == srvT.CLIENT)
			runClient();
	}


	/**
	 * 
	 */
	private void runServer() {
		relayT		relay; 
		sysMsgT		msg;
		boolean	curState;

		//
		// running as a server
		//
		while (true) {

			msg = sysMsgT.readMsg();
			if (msg == null)
				break;

			switch (msg.cmd) {
				case sysMsgT.MSG_CONNECTION:
					Socket s = (Socket)msg.obj1;
					new clientConnectionT(m_srv,s);
					m_srv.display("socket connection:"+s);
					break;
                    
				case sysMsgT.MSG_OVERRIDE:
					relay = (relayT)msg.obj1;
					if (msg.int1 == 1)
						relay.setOverRide(true);
					else if (msg.int1 == 0) 
						relay.setOverRide(false);
					else if (msg.int1 == -1)
						relay.setOverRideOff();
					clientConnectionT.overrideRelay(relay, msg.int1);
					break;
					

				case sysMsgT.MSG_PROC_SENSOR:
					sensorT sensor = (sensorT)msg.obj1;
					int dat = sensor.getSensorDat();
					sensor.process();
					m_srv.ui.updSensor(sensor);
					if (dat != sensor.getSensorDat()) {
						sensor.updCharts();
						clientConnectionT.procSensor(sensor);
						sensor.updSql();
					}
					break;

				case sysMsgT.MSG_PROC_RELAY:
					relay = (relayT)msg.obj1;
					curState = relay.getState();
					relay.process();
					m_srv.ui.updRelay(relay);
					if (curState != relay.getState()) {
						clientConnectionT.procRelay(relay);
					}
					break;

				case sysMsgT.MSG_EDIT_RELAY:
					relayT newRelay = (relayT)msg.obj1;
					relayT oldRelay = (relayT)msg.obj2;
					
					clientConnectionT.editRelay(oldRelay,newRelay);
					if (oldRelay == null) {
						deviceT.addToList(newRelay); 
						m_srv.ui.updRelay(newRelay);
					}
					else {
						oldRelay.update(newRelay);
						m_srv.ui.updRelay(oldRelay);
					}											
					m_srv.ini.saveProp();
					break;
					
				case sysMsgT.MSG_DEL_RELAY:
					deviceT.del((String)msg.obj1);
					m_srv.ui.delRelay((String)msg.obj1);
					m_srv.ini.saveProp();
					clientConnectionT.delRelay((String)msg.obj1);
					break;
					
				case sysMsgT.MSG_CLR_SENSOR:
					sensor = (sensorT)msg.obj1;
					if (sensor != null) {
						sensor.clrSql();
						m_srv.ui.updSensor(sensor);
					}
					break;
					
				case sysMsgT.MSG_DEL_SENSOR:
					deviceT.del((String)msg.obj1);
					m_srv.ui.delSensor((String)msg.obj1);
					m_srv.ini.saveProp();
					clientConnectionT.delSensor((String)msg.obj1);
					break;

				case sysMsgT.MSG_EDIT_SENSOR:
					sensorT newSensor = (sensorT)msg.obj1;
					sensorT oldSensor = (sensorT)msg.obj2;
					
					//clientConnectionT.editSensor(oldSensor,newSensor);
					clientConnectionT.editSensor(newSensor,oldSensor);
					if (oldSensor == null) {
						deviceT.addToList(newSensor);
						m_srv.ui.updSensor(newSensor);
					}
					else {
						oldSensor.update(newSensor);
						m_srv.ui.updSensor(oldSensor);
					}					 
					m_srv.ini.saveProp();
					break;

				case sysMsgT.MSG_EDIT_BOARD:
					ioPortT io = (ioPortT)msg.obj1;
					io.setBase(msg.int1);
					io.setEnabled((msg.int2==1) ? true:false);
					m_srv.ui.updController(io);
					m_srv.ini.saveProp();
					clientConnectionT.editBoard(io);
					break;
			}
            
			sysMsgT.releaseMsg(msg);
		}
	}


	/**
	 * 
	 */
	private void runClient() {
		msgT 	msg;
		ioPortT	io;
		sysMsgT smsg;
		relayT	relay;
		sensorT	sensor;

		//
		// running as a client
		//
		while (true) {

			smsg = sysMsgT.readMsg();
			if (smsg == null)
				break;

			switch (smsg.cmd) {
				case sysMsgT.MSG_OVERRIDE:
					relay = (relayT)smsg.obj1;
					msg = new msgT(sysMsgT.MSG_OVERRIDE, relay.getName());
					msg.int1 = smsg.int1;
					m_srv.sock.sendMsg(msg);
					break;
				
				case sysMsgT.MSG_EDIT_RELAY:
					relayT newRelay = (relayT)smsg.obj1;
					relayT oldRelay = (relayT)smsg.obj2;
					msg = new msgT(); 
					msg.bldRelayMsg(newRelay);
					if (msg.board.equals("Parallel"))
						msg.board = "pport";
					if (oldRelay != null) {
						msg.state = oldRelay.getState() ? 1:0;
						if (oldRelay.getName().equals(newRelay.getName()) == false)
							msg.data4 = oldRelay.getName();
					}
					m_srv.sock.sendMsg(msg);
					break;
					
				case sysMsgT.MSG_DEL_RELAY:
					// client wants to delete a relay; tell server
					m_srv.sock.sendMsg(new msgT(sysMsgT.MSG_DEL_RELAY, (String)smsg.obj1));
					break;
					
				case sysMsgT.MSG_DEL_SENSOR:
					// client wants to delete a sensor; tell server
					m_srv.sock.sendMsg(new msgT(sysMsgT.MSG_DEL_SENSOR, (String)smsg.obj1));
					break;

				case sysMsgT.MSG_EDIT_SENSOR:
					sensorT newSensor = (sensorT)smsg.obj1;
					sensorT oldSensor = (sensorT)smsg.obj2;
					msg = new msgT();
					msg.bldSensorMsg(newSensor);					
					if (oldSensor != null) {
						if (oldSensor.getName().equals(newSensor.getName()) == false)
							msg.data4 = oldSensor.getName();
					}
					m_srv.sock.sendMsg(msg);
					break;

				case sysMsgT.MSG_EDIT_BOARD:
					io = (ioPortT)smsg.obj1;
					msg = new msgT();
					msg.cmd = sysMsgT.MSG_EDIT_BOARD;
					msg.board = io.getName();
					msg.int1 = smsg.int1;		//addr
					msg.int2 = smsg.int2;		//enabled flg
					m_srv.sock.sendMsg(msg);
					break;
					
				case sysMsgT.MSG_CLIENT:
					msg = (msgT)smsg.obj1;
					switch (msg.cmd) {
						case sysMsgT.MSG_OVERRIDE:
							//
							// msg.device == device name
							// msg.state  == state
							// msg.int1   == override state
							//
							relay = (relayT)deviceT.fnd(msg.device);
							if (relay != null) {
								relay.setState(msg.state == 1);
								switch (msg.int1) {
									case -1:
										relay.setOverRideOff();
										break;
									case 0:
										relay.setOverRide(false);
										break;
									case 1:
										relay.setOverRide(true);
										break;
								}
								m_srv.ui.updRelay(relay);
							}
							break;


						case sysMsgT.MSG_PROC_RELAY:
							//
							// msg.device == device name
							// msg.state  == state
							//
							relay = deviceT.fndRelay(msg.device);
							if (relay != null) {
								relay.setState(msg.state == 1);
								m_srv.ui.updRelay(relay);
							}
							break;

						case sysMsgT.MSG_PROC_SENSOR:
							//
							// msg.device == device name
							// msg.int1   == sensor data
							//
							sensor = deviceT.fndSensor(msg.device);
							if (sensor != null) {
								sensor.setSensorDat(msg.int1);
								m_srv.ui.updSensor(sensor);
								sensor.updCharts();
							}
							break;


						case sysMsgT.MSG_EDIT_RELAY:
							if (msg.board.equals("Parallel"))
								msg.board = "pport";
							relay = relayT.loadRelay(msg);
							if (relay != null)
								m_srv.ui.updRelay(relay);
							break;
					
						case sysMsgT.MSG_DEL_RELAY:
							deviceT.del(msg.device);
							m_srv.ui.delRelay(msg.device);
							break;

					
						case sysMsgT.MSG_DEL_SENSOR:
							deviceT.del(msg.device);
							m_srv.ui.delSensor(msg.device);
							break;

						case sysMsgT.MSG_EDIT_SENSOR:
							sensor = sensorT.loadSensor(msg);
							if (sensor != null)
								m_srv.ui.updSensor(sensor);
							break;

						case sysMsgT.MSG_EDIT_BOARD:
							io = (ioPortT)ioPortT.fnd(msg.board);
							if (io != null) {
								io.setBase(msg.int1);
								io.setEnabled((msg.int2==1) ? true:false);
								m_srv.ui.updController(io);
							}
							break;
					}
					break;
			}
            
			sysMsgT.releaseMsg(smsg);
		}
	}
	
}

