/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.srv;

import JAuto.util.*;
import JAuto.ui.*;

import java.util.*;


public class relayT extends deviceT {
	public	static	final	int	CLOCK = 0;
	public	static	final	int	CYCLE = 1;
	public	static	final	int	SLAVE = 2;
	public	static	final	int	COND  = 3;

	private	int		relayTyp;
	private	Date		nextEvent;		//used by CYCLE type relays
	private	boolean	cycleState;		//used by CYCLE type relays

	private	boolean	overRide;
	private	boolean	overRideState;

	private	LinkedList	onConditions;
	private	LinkedList	offConditions;
	
	private	Object		condSyncObj = new Object();
	private	LinkedList	charts = new LinkedList();

	/**
	 * copy constructor
	 * 
	 * @param relay
	 */
	public relayT(relayT relay) {
		super(relay);

		this.relayTyp      = relay.relayTyp;
		this.onSecs        = relay.onSecs;
		this.offSecs       = relay.offSecs;
		this.nextEvent     = relay.nextEvent;
		this.cycleState    = relay.cycleState;

		this.overRide      = relay.overRide;
		this.overRideState = relay.overRideState;

		this.onConditions  = relay.onConditions;
		this.offConditions = relay.offConditions;
	}


	/**
	 * 
	 * @param name
	 */
	public relayT(String name) {
		super(name, deviceT.TYP_RELAY);
		
		this.onSecs = 0;
		this.offSecs = 0;
		this.nextEvent = new Date();
		this.cycleState = false;

		this.onConditions = new LinkedList();
		this.offConditions = new LinkedList();
	}


	/**
	 * Create a relay instance.
	 *
	 * @param name			name of this relay.
	 * @param relayTyp		type of relay (CLOCK, CYCLE, SLAVE, COND).
	 * @param ioBoard		controller this relay is on.
	 * @param channel		channel on controller for this relay.
	 */
	public relayT(String name, int relayTyp, ioPortT ioBoard, int channel) {
		super(name, deviceT.TYP_RELAY);
		super.setBoard(ioBoard);
		super.setChannel(channel);

		this.relayTyp = relayTyp;
		this.onSecs = 0;
		this.offSecs = 0;
		this.nextEvent = new Date();
		this.cycleState = false;

		this.onConditions = new LinkedList();
		this.offConditions = new LinkedList();
	}


	/**
	 * 
	 * @param relay
	 */
	public void update(relayT relay) {
		setName(relay.getName());
		setBoard(relay.getBoard());
		setChannel(relay.getChannel());
		setState(relay.getState());
		
		this.onTime      = relay.onTime;
		this.onCalendar  = relay.onCalendar;
		this.offTime     = relay.offTime;
		this.offCalendar = relay.offCalendar;
		this.slaveOf     = relay.slaveOf;
		
		this.relayTyp = relay.relayTyp;
		
		setOnConditions(relay.onConditions.iterator());
		setOffConditions(relay.offConditions.iterator());
	}


	/**
	 * set the type of this relay
	 * 
	 * @param typ
	 */	
	public void setRelayType(int typ) {
		relayTyp = typ;
	}

	/**
	 * set the type of this relay
	 * 
	 * @param typ
	 */	
	public void setRelayType(String typ) {
		if (typ.equalsIgnoreCase("clock"))
			relayTyp = CLOCK;
		else if (typ.equalsIgnoreCase("cycle"))
			relayTyp = CYCLE;
		else if (typ.equalsIgnoreCase("slave"))
			relayTyp = SLAVE;
		else if (typ.equalsIgnoreCase("cond"))
			relayTyp = COND;
		else
			srv.display("invalid relay type:"+typ);
	}

	/**
	 * set state of relay based on/off times of this relay
	 */
	public void setStateByOnOffTime() {
		if (chkOnOffTime()) 
			setState(true);
		else 
			setState(false);
	}


	/**
	 * set on/off state of this relay
	 * 
	 * @see deviceT#setState(boolean)
	 */
	public void setState(boolean flg) {
		if (getState() == flg)
			return;
			
		if (flg)
			on();
		else
			off();
			
		super.setState(flg);
		
		updSql();
	}
	
	/**
	 * update the sql database with relay state
	 */
	public void updSql() {
		if (srv.ini.sqlDetail) {
			int state = (getState() == true) ? 1:0;
			srv.sql.update("INSERT INTO relayLog (ts,name,state) "+
							"values (NOW(),"+
							"'"+name+"'," +						
							"'"+state+"')");						
		}
	}	

	
	/**
	 * set the on/off time of this relay
	 * 
	 * @param onOff
	 * @param h
	 * @param m
	 * @param s
	 */	
	public void setTime(int onOff, int h, int m, int s) {
		if (relayTyp == CLOCK) {
			super.setTime(onOff,h,m,s);
			return;
		}
		
		if (onOff == deviceT.OFF) {
			offCalendar = Calendar.getInstance();
			offCalendar.clear();
			offCalendar.set(Calendar.DAY_OF_MONTH, 1);
			offCalendar.set(Calendar.HOUR_OF_DAY, h);
			offCalendar.set(Calendar.MINUTE, m);
			offCalendar.set(Calendar.SECOND, s);
			this.offTime = offCalendar.getTime();
			this.offSecs = (h*3600) + (m*60) + s;
		}
		else {
			onCalendar = Calendar.getInstance();
			onCalendar.clear();
			onCalendar.set(Calendar.DAY_OF_MONTH, 1);
			onCalendar.set(Calendar.HOUR_OF_DAY, h);
			onCalendar.set(Calendar.MINUTE, m);
			onCalendar.set(Calendar.SECOND, s);
			this.onTime = onCalendar.getTime();
			this.onSecs = (h*3600) + (m*60) + s;
		}
	}

	
	/**
	 * turn relay on
	 */
	private void on() {
		if (board instanceof B2102) {
			B2102 b2102 = (B2102)board;
			b2102.setState(channel,1);
			srv.display("turning on B2102 relay:"+getName()+
						" nextEvent:"+nextEvent);
		}
		else if (board instanceof pportT) {
			pportT pport = (pportT)board;
			pport.setState(channel,true);
			srv.display("turning on PPORT relay:"+getName()+
						" nextEvent:"+nextEvent);			
		}		
	}
	
	
	/**
	 * turn relay off
	 */
	private void off() {
		if (board instanceof B2102) {
			B2102 b2102 = (B2102)board;
			b2102.setState(channel,0);
			srv.display("turning off B2102 relay:"+getName()+
						" nextEvent:"+nextEvent);
		}
		else if (board instanceof pportT) {
			pportT pport = (pportT)board;
			pport.setState(channel,false);
			srv.display("turning off PPORT relay:"+getName()+
						" nextEvent:"+nextEvent);
		}

	}


	/**
	 * Returns the relayTyp.
	 * 
	 * @return int
	 */
	public int getRelayTyp() {
		return relayTyp;
	}


	/**
	 * get the relay type string
	 * 
	 * @return String
	 */
	public String getRelayTypStr() {
		switch (relayTyp) {
			case CLOCK:	return "clock";
			case CYCLE:	return "cycle";
			case SLAVE:	return "slave";
			case COND:		return "cond";
		}
		return "?????";
	}

	/**
	 * 
	 * @param chart
	 */
	public void addChart(relayChartT chart) {
		charts.add(chart);
	}

	public void delChart(relayChartT chart) {
		charts.remove(chart);
	}
	
	public void updCharts() {
		int	 flg = (getState()==true) ? 1:0;
		Iterator itr = charts.iterator();
		
		while (itr.hasNext()) {
			relayChartT chart = (relayChartT)itr.next();					 
			chart.addToSeries(this, flg);
		}
	}



	/**
	 * get the "on condition" list
	 * 
	 * @return LinkedList
	 */
	public LinkedList getOnConditions() {
		return onConditions;
	}


	/**
	 * Add an "on" condition to the device.
	 *
	 * @param condition	the <code>condT</code> object to add
	 */
	public void addOnCondition(condT condition) {
		synchronized (condSyncObj) {
			onConditions.add(condition);
		}
	}


	/**
	 * set the "on condition" list
	 */
	public void setOnConditions(Iterator itr) {
		synchronized (condSyncObj) {
			onConditions.clear();
			while (itr.hasNext()) {
				condT cond = (condT)itr.next();
				onConditions.add(cond);
			}
		}
	}


	/**
	 * get the "off condition" list
	 * 
	 * @return LinkedList
	 */
	public LinkedList getOffConditions() {
		return offConditions;
	}


	/**
	 * Add an "off" condition to the device.
	 *
	 * @param condition	the condT object to add
	 */
	public void addOffCondition(condT condition) {
		synchronized (condSyncObj) {
			offConditions.add(condition);
		}
	}
	
	
	/**
	 * set the "off condition" list
	 */
	public void setOffConditions(Iterator itr) {
		synchronized (condSyncObj) {
			offConditions.clear();
			while (itr.hasNext()) {
				condT cond = (condT)itr.next();
				offConditions.add(cond);
			}
		}
	}

	
	/**
	 * chk conditions based on current state of relay
	 * 
	 * @return boolean
	 */
	private boolean chkConditionsCurState() {
		if (getState() == true) 
			return chkConditions(deviceT.ON);
		return chkConditions(deviceT.OFF);
	}
	
	
	/**
	 * chk conditions of this relay
	 * 
	 * @return boolean
	 */
	private boolean chkConditions(int onOff) {
		
		Iterator itr;
		
		synchronized (condSyncObj) {
			if (onOff == deviceT.ON) 
				itr = onConditions.iterator();
			else
				itr = offConditions.iterator();
		
			while (itr.hasNext()) {
				condT cond = (condT)itr.next();
				if (cond.chk() == false)
					return false;			
			}
		}
		return true;		
	}


	/**
	 * Returns the overRide.
	 * @return boolean
	 */
	public boolean isOverRide() {
		return overRide;
	}

	
	/**
	 * 
	 */
	public void setOverRide(boolean flg) {
		overRide = true;
		overRideState = flg;
	}


	/**
	 * 
	 */	
	public void setOverRideOff() {
		overRide = false;
	}

	
	/**
	 * process this relay
	 * 
	 * @see deviceT#process()
	 */
	public void process() {
		
		if (overRide) {
			setState(overRideState);
			return;
		}


		switch (relayTyp) {
			case CLOCK:
				boolean inWindow = chkOnOffTime();
				if (getState() == true) {
					//
					// device is currently on
					//
					if (onConditions.size() > 0  && chkConditions(deviceT.ON)==true) {
						// conditions are TRUE; turn relay off; no time window chk
						setState(false);
					}
					else {
						// conditions failed; set state based on time window
						setState(inWindow);
					}
				}
				else {
					//
					// device is currently off; if not in time window, do nothing
					//
					if (inWindow == false) {
						setState(false);
					}
					else {
						if (offConditions.size() > 0) {
							if (chkConditions(deviceT.OFF)==true) {
								// conditions are TRUE; turn relay on 
								setState(true);
							}
							else {
								setState(false);
							}
						}
						else {
							setState(true);
						}
					}
				}
				break;
				
				
			case CYCLE:
				if (srv.curTime.compareTo(nextEvent) >= 0) {
					if (getState() == true) {
						//
						// device is on; turn it off
						//
						nextEvent.setTime(srv.curTime.getTime() + (offSecs*1000));
						cycleState = false;
					}
					else {
						//
						// device is off; turn it on
						//
						nextEvent.setTime(srv.curTime.getTime() + (onSecs*1000));
						cycleState = true;
					}
				}
				
				if (cycleState == false) {
					setState(false);
				}
				else {
					if (onConditions.size() > 0) {
						if (chkConditions(deviceT.ON)==true) {
							// conditions passed
							setState(false);
						}
						else {
							// conditions failed
							setState(true);
						}
					}
					else {
						setState(true);
					}					 
				}
				break;

			
			case SLAVE:
				if (slaveOf != null) {
					boolean masterState = slaveOf.getState();					
					if (masterState == true) {
						//
						// master device is currently on
						//
						if (onConditions.size() > 0  && chkConditions(deviceT.ON)==true) {
							// conditions are TRUE; turn relay off
							masterState = false;
						}
					}
					setState(masterState);
				}
				break;

				
			case COND:
				if (getState() == true) {
					//
					// device is currently on
					//
					if (onConditions.size() > 0  && chkConditions(deviceT.ON)==true) {
						// conditions are TRUE; turn relay off
						setState(false);
					}
				}
				else {
					//
					// device is currently off
					//
					if (offConditions.size() > 0 && chkConditions(deviceT.OFF)==true) {
						// conditions are TRUE; turn relay on 
						setState(true);
					}
				}
				break;
		}
		
		updCharts();


	}
	
	
	/**
	 * create a relayT from a msgT
	 * 
	 * @param msg
	 * 
	 * @return relayT
	 */
	public static relayT loadRelay(msgT msg) {
		relayT	relay;
		String	name;

		if (msg.data4.length() == 0)
			name = msg.device;
		else
			name = msg.data4;

		relay = deviceT.fndRelay(name);
		if (relay == null) {
			relay = new relayT(msg.device);			
			deviceT.addToList(relay);
		}

		relay.setName(msg.device);
		relay.setBoard(msg.board);
		relay.setChannel(msg.channel);
		relay.setRelayType(msg.int1);
		
		if (msg.int1 == relayT.SLAVE)
			relay.setSlaveDevice(msg.slaveOf);
		else {
			StringTokenizer st;
			if (msg.int1 != relayT.COND) {
				st = new StringTokenizer(msg.data1,":");
				if (st.hasMoreTokens()) {
					String hours = st.nextToken();
					String mins = st.nextToken();
					String secs = st.nextToken();				 
					relay.setTime(1,	
								  Integer.parseInt(hours), 
								  Integer.parseInt(mins),
								  Integer.parseInt(secs));

					hours = st.nextToken();
					mins = st.nextToken();
					secs = st.nextToken();				 
					relay.setTime(0,	
								  Integer.parseInt(hours), 
								  Integer.parseInt(mins),
								  Integer.parseInt(secs));
				}
			}
			
			relay.getOnConditions().clear();
			st = new StringTokenizer(msg.data2,"|");
			while (st.hasMoreTokens()) {
				String tok = st.nextToken();
				StringTokenizer st1 = new StringTokenizer(tok,":");
				if (st1.hasMoreTokens()) {	
					String device = st1.nextToken();
					String op	  = st1.nextToken();
					String val	  = st1.nextToken();
					
					relay.addOnCondition( new condT(device, op, val) );					
				}
			}
			
			relay.getOffConditions().clear();
			st = new StringTokenizer(msg.data3,"|");
			while (st.hasMoreTokens()) {
				String tok = st.nextToken();
				StringTokenizer st1 = new StringTokenizer(tok,":");
				if (st1.hasMoreTokens()) {	
					String device = st1.nextToken();
					String op	  = st1.nextToken();
					String val	  = st1.nextToken();
					
					relay.addOffCondition( new condT(device, op, val) );					
				}
			}
		}

		relay.setState(msg.state == 1); 
		if (msg.int2 == 0)
			relay.setOverRideOff();
		else {
			relay.setOverRide(msg.int2 == 1);
		}
		
		return relay;
	}

}
