/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.srv;

import JAuto.util.*;
import JAuto.ui.*;

import java.util.*;

public class sensorT extends deviceT {
	public	static	final	int	VOLTAGE = 0;
	
	public		double		avg=0;	
	
	private	int		sensorDat;
	private	double		multiplier = 1.0;
	private	LinkedList	charts = new LinkedList();	

	/**
	 * Constructor for sensorT.
	 * 
	 * @param name
	 * @param typ
	 */
	public sensorT(String name) {
		super(name, deviceT.TYP_INPUT);
		
		this.sensorDat = 0;
	}


	/**
	 * Constructor for sensorT.
	 * 
	 * @param name
	 * @param typ
	 */
	public sensorT(String name, ioPortT ioBoard, int channel) {
		super(name, deviceT.TYP_INPUT);
		super.setBoard(ioBoard);
		super.setChannel(channel);
		
		this.sensorDat = 0;
	}

	/**
	 * 
	 * @param sensor
	 */
	public void update(sensorT sensor) {
		setName(sensor.getName());
		setBoard(sensor.getBoard());
		setChannel(sensor.getChannel());
	}

	/**
	 * Get the last read data for this sensor.
	 */
	public int getSensorDat() {
		return sensorDat;
	}


	/**
	 * Sets the sensorDat.
	 * @param sensorDat The sensorDat to set
	 */
	public void setSensorDat(int sensorDat) {
		this.sensorDat = sensorDat;
	}
	
	
	/**
	 * 
	 * @param chart
	 */
	public void addChart(sensorChartT chart) {
		charts.add(chart);
	}

	public void delChart(sensorChartT chart) {
		charts.remove(chart);
	}
	
	public void updCharts() {
		Iterator itr = charts.iterator();
		while (itr.hasNext()) {
			sensorChartT chart = (sensorChartT)itr.next();
					 
			chart.addToSeries(this, sensorDat);
		}
	}


	/**
	 * clear the average sensor value in the sql database
	 */
	public void clrSql() {
		avg = sensorDat;
		srv.sql.update(
					"UPDATE sensorAvg SET "+
					"count='1',value='"+sensorDat+"',ts=NOW() WHERE name='"+name+"'");
	}


	/**
	 * update the sql database with sensor value
	 */
	public void updSql() {
		long		count;
		double		val;
		sqlResultT	result;

		result = srv.sql.query(
					"SELECT count,value FROM sensorAvg WHERE name='" + name + "'");

		if (result.hasNext() == false) {
			/*
			 * new entry
			 */
			srv.sql.update(
					"INSERT INTO sensorAvg " +
					"(name,ts,value) VALUES (" +					
					"'" + name + "',NOW()," +
					"'" + sensorDat + "')");
			avg = sensorDat;
		}
		else {
			/*
			 * update existing entry
			 */
			count = result.getLong(0);
			val = result.getDouble(1);			
			avg = (val * count) + sensorDat;
			
			count++;
			avg /= count;
			srv.sql.update(
						"UPDATE sensorAvg SET "+
						"count='"+count+"',"+
						"value='"+avg+"',"+
						"ts=NOW() " +
						"WHERE name='"+name+"'");
		}
		
		
		if (srv.ini.sqlDetail) {		
			srv.sql.update("INSERT INTO sensorLog (ts,name,value) "+
							"values (NOW(),"+
							"'"+name+"'," +						
							"'"+sensorDat+"')");
		}
	}

	/**
	 * process this sensor
	 * 
	 * @see deviceT#process()
	 */	
	public void process() {
		if (board instanceof B3001) {
			if (srv.b3001.getEnabled()) {
				B3001 b3001 = (B3001)board;
				sensorDat = b3001.readChannel(channel);
			}
		}
	}


	/**
	 * 
	 * @param msg
	 * 
	 * @return sensorT
	 */	
	public static sensorT loadSensor(msgT msg) {
		sensorT	sensor;
		String	name;

		if (msg.data4.length() == 0)
			name = msg.device;
		else
			name = msg.data4;

		sensor = deviceT.fndSensor(name);
		if (sensor == null) {
			sensor = new sensorT(msg.device);			
			deviceT.addToList(sensor);
		}

		sensor.setName(msg.device);
		sensor.setBoard(msg.board);
		sensor.setChannel(msg.channel);
		sensor.setSensorDat(msg.int1);

		return sensor;
	}

	/**
	 * Returns the multiplier.
	 * @return int
	 */
	public double getMultiplier() {
		return multiplier;
	}

	/**
	 * Sets the multiplier.
	 * @param multiplier The multiplier to set
	 */
	public void setMultiplier(double multiplier) {
		this.multiplier = multiplier;
	}

	/**
	 * Returns the avg.
	 * @return double
	 */
	public int getAvg() {
		return (int)avg;
	}

}
