/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.srv;

import JAuto.util.*;
import JAuto.ui.*;

import java.util.*;
import javax.swing.*;

public class srvT extends Thread {
	public	static	final	int	SERVER = 0;
	public	static	final	int	CLIENT = 1;
		
	static	srvT		instance = null;
	
	public	uiT			ui;
	public	boolean	runThread = true;
	public	B2102		b2102;
	public	B3001		b3001;
	public	pportT		pport;

	public	Calendar	curCalendar;
	public	Date		curTime;
	public	iniT		ini;
	public	int		appType;
	public	String		serverName;
	public	String		iniName;
	public	msgThreadT	msgThread;
	public	String		user;
	public	String		pw;
	
	public	sockT 		sock;
	public	sqlT		sql;
	

	/**
	 * create a server instance
	 */
	public srvT(int appType) {
		instance = this;
		
		this.appType = appType;
//		this.serverName = serverName;
//		this.iniName = iniName;

		msgThread = new msgThreadT(this);
		msgThread.start();		

		b2102 = new B2102("b2102");
		b3001 = new B3001("b3001");
		pport = new pportT("pport");
		
		sql = new sqlT(this);		
	}

	
	/**
	 * init things
	 */
	private void init() {
		String ttl;
		if (appType == SERVER) 
			ttl = "Java Automation - v1.5.1, Server mode";
		else
			ttl = "Java Automation - v1.5.1, Client mode";
		
		ini = new iniT();
		ini.loadSqlSettings();
		
		ui = new uiT(ttl);

		//
		// load ini settings
		//
		if (appType == srvT.SERVER) {
			if (ioPortT.jniLoaded == false) {
				System.out.println("java.library.path = " + System.getProperty("java.library.path")+"\n");
				JOptionPane.showMessageDialog(null,"Error loading 'ioPortT' library");
				System.exit(-1);
			}
						
			ini.loadProp();
		
			//
			// setup io controllers
			//
			if (b2102.getEnabled())
				b2102.open();		//default base=0x03E0, delay=100us
			
			if (b3001.getEnabled())
				b3001.open();		//default base=0x03E4, delay=100us
			
			if (pport.getEnabled())		
				pport.open();		//default base=0x0378, delay=100us
		}
		
		ui.updController(b2102);
		ui.updController(b3001);
		ui.updController(pport);
		
		/*
		 * connect to sql database (if avail)
		 */
		sql.connect(ini.sqlHost, ini.sqlDB, ini.sqlUser, ini.sqlPW);
	}


	/**
	 * display a msg to the msg tab on the server
	 * 
	 * @param msg
	 */
	public void display(String msg) {
		//System.out.println(msg);
		ui.logMsg(msg);
	}


	/**
	 * stop the server
	 */	
	public void shutdown() {
		runThread = false;
		interrupt();
		
		b2102.close();
		b3001.close();
		pport.close();

		ui.dispose();
		System.exit(0);
	}
	
	
	/**
	 * main processing thread
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		if (appType == srvT.SERVER)
			runServer();
		else if (appType == srvT.CLIENT)
			runClient();
	}
	

	/**
	 * 
	 */	
	private void runServer() {
		
		new serverThreadT(this);
		
		//
		// do the work
		//
		long	loop = 0;
		while (runThread) {
			Iterator	itr;

			curCalendar = Calendar.getInstance();
			curTime = curCalendar.getTime(); 
			
			//
			// read sensors once every 3 seconds
			//
			if ((loop++ % 3) == 0) {
				synchronized (deviceT.class) {
					itr = deviceT.sensors.iterator();
					while (itr.hasNext()) {
						sysMsgT.postMsg(sysMsgT.MSG_PROC_SENSOR, itr.next());
					}
				}
			}
			
			//
			// process relays
			//
			synchronized (deviceT.class) {
				itr = deviceT.relays.iterator();
				while (itr.hasNext()) {
					sysMsgT.postMsg(sysMsgT.MSG_PROC_RELAY, itr.next());
				}
			}
			
			//
			// sleep for 1 second
			//
			try { sleep(1000); }
			catch (Exception e) {
				runThread = false;
			}
		}

	}
	
	/**
	 * 
	 */	
	private void runClient() {
		if (serverName == null)
			return;
		
		sock = new sockT();
		if (sock.connect(serverName,9189) == -1) {
			return;
		}
		
		msgT msg = new msgT();
		msg.cmd = sysMsgT.MSG_LOGON;
		msg.data1 = user;
		msg.data2 = pw;
		sock.sendMsg(msg);

		while (runThread) {
			//
			// read msg from server
			//
			msg = new msgT();
			if (sock.readMsg(msg) < 0) {
				runThread = false;
				continue;
			}
						
			sysMsgT.postMsg(sysMsgT.MSG_CLIENT, msg);
						
		}

	}

	/**
	 * main entry int to this app
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		String	iniName = "./jauto.ini";
		String	srvName = null;
		String	user = "";
		String	pw = "";
		int	appTyp = srvT.SERVER;
		
		
		for (int i=0; i<args.length; i++) {		
			String	arg = args[i];			
			
			if (arg.charAt(0) == '-') {
				switch (arg.charAt(1)) {
					case 'u':
						user = args[i].substring(2);
						break;
						
					case 'p':
						pw = args[i].substring(2);
						break;
						
					case 's':
						srvName = args[i].substring(2);
						appTyp = srvT.CLIENT;
						break;
					
					case 'i':
						iniName = args[i].substring(2);
						break;
				}
			}
		}
		
		srvT server = new srvT(appTyp);

		//server.appType = appTyp;
		server.serverName = srvName;
		server.iniName = iniName;
		server.user = user;
		server.pw = pw;
		
		server.init();
		server.start();
		
		try { server.join();}
			catch (Exception e) {}
	}
	
	
	/**
	 * get the instance of this server
	 * 
	 * @return srvT
	 */
	public static srvT getInstance() {
		return instance;
	}
}
