/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.ui;

import JAuto.srv.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;


/**
 * condition table model
 */
class conditionTblModelT extends tableModelT {
	static final	int	DEVICE	= 0;
	static final	int	OP		= 1;
	static final	int 	VALUE	= 2;
	static final   int	COLUMN_CNT = VALUE+1;

	relayTblT	tbl;	
	String		headers[] = {"Device","","Value"};
	
	/**
	 * Constructor for conditionTblModelT.
	 */
	public conditionTblModelT() {
		super();
	}

	public int getColumnCount()                 { return COLUMN_CNT; }
	public String getColumnName(int col)        { return headers[col]; }
	
	/**
	 * 
	 */
	public Object getValueAt(int row, int col) {
		condT	cond = (condT)rowData.elementAt(row);
		
		switch (col) {
			case DEVICE:	return cond.getDevice();
			case OP:		return cond.getOpString();
			case VALUE:	return ""+cond.getVal();
		}
		return "?";
	}
	

	/**
	 * add a condition to the tbl
	 * 
	 * @param relay
	 */
	public void add(condT cond) {
		if (cond == null)
			return;
		
		if (fnd(cond) == null) {
			rowData.add(cond);
			fireInserted(getRowCount()-1);
			calcRowNumers();
		}
	}
	
	/**
	 * update a condition in the tbl
	 * 
	 * @param relay
	 */
	public void upd(condT cond) {
		if (cond == null)
			return;
		
		if (fnd(cond) != null) {
			fireUpd(cond.getTblRow());
		}
	}


	/**
	 * delete a condition from the tbl
	 * 
	 * @param relay
	 */
	public void delete(condT cond) {
		if (cond == null)
			return;
			
		condT c = fnd(cond);		
		if (c != null) {
			rowData.remove(c);
			fireDeleted(c.getTblRow());
		}
	}
	
	
	/**
	 * find a condition in the tbl
	 * 
	 * @param name
	 * 
	 * @return condT
	 */
	condT fnd(condT cond) {
		Iterator itr = iterator();
		
		while (itr.hasNext()) {
			condT c = (condT)itr.next();
			if (cond == c)
				return c;
		}
		return null;
	}

	
	/**
	 * recalc row numbers for tbl display
	 * 
	 * @param name
	 * 
	 * @return condT
	 */
	void calcRowNumers() {
		int		row=0;
		Iterator 	itr = iterator();

		while (itr.hasNext()) {
			condT relay = (condT)itr.next();
			relay.setTblRow(row++);
		}
	}
}



/**
 * condition table
 */
public class conditionTblT extends JTable {
	
	conditionTblModelT	mdl;
	
	public conditionTblT(conditionTblModelT mdl) {
		super(mdl);
		this.mdl = mdl;

		setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		JTableHeader myHeader = getTableHeader();
		myHeader.setBorder(BorderFactory.createRaisedBevelBorder());
		
		addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				Point   p = e.getPoint();
				int     row = rowAtPoint(p);

				getSelectionModel().setSelectionInterval(row,row);
				if (e.getClickCount() == 2) {
					//
					// hdl dbl click
					//
					dblClick(row);
				}
			}
		});
		
		TableColumn  column = null;
		try {
			// operation col
			column = getColumnModel().getColumn(conditionTblModelT.OP);
			column.setCellRenderer(new tblCellRenderT());
			column.setPreferredWidth(30);

			// value col
			column = getColumnModel().getColumn(conditionTblModelT.VALUE);
			column.setCellRenderer(new tblCellRenderT());
			
		}
		catch (Exception e) {}


	}
	
	Iterator iterator() { return mdl.rowData.iterator(); }	
	public void add(condT cond) { mdl.add(cond); }
	public void upd(condT cond) { mdl.upd(cond); }
	public void delete(condT cond) { mdl.delete(cond); }
	
	
	/**
	 * get the condition at 'row' in the tbl
	 * 
	 * @param row
	 * 
	 * @return condT
	 */
	public condT getEntry(int row) {
		try {
			if (row < 0)
				row = getSelectedRow();
			else if ( row > getRowCount())
				return null;

			return (condT)mdl.rowData.elementAt(row);
		}
		catch (Exception excp) {
			return null;
		}
	}
	
	
	/**
	 * handle a double click in the tbl
	 * 
	 * @param row
	 */
	public void dblClick(int row) {
		condT cond = getEntry(row);

		if (cond != null) {
			new condEditT(this,cond);
		}
	}		
}
