/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.ui;

import JAuto.srv.*;
import JAuto.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;
import java.text.*;
import java.util.*;


public class relayEditT extends frameT {
	srvT 			srv = srvT.getInstance();
	uiT 			ui = uiT.getInstance();
	
	relayT 			relay;
	conditionPnlT	onTab;
	conditionPnlT	offTab;
	JTabbedPane		tabbedPane = new JTabbedPane();
	
	JPanel			appPnl = new JPanel();
	JPanel			appTop = new JPanel();
	JPanel			header1 = new JPanel();
	JPanel			header2 = new JPanel();
	JPanel			header3 = new JPanel();
	JPanel			appBottom = new JPanel();
	JComboBox		cbDev = new JComboBox();
	JComboBox 		cbType = new JComboBox();
	JComboBox 		cbBoard = new JComboBox();
	JComboBox 		cbChannel = new JComboBox();
	JTextField		fldName;
	JFormattedTextField		fldTimeOn;
	JFormattedTextField		fldTimeOff;
	JLabel					lblTimeOn = new JLabel("Time on:");
	JLabel					lblTimeOff = new JLabel("Time off:");
	JLabel					lblSlaveOf = new JLabel("Slave of:");
	
	JButton			okButton = new JButton("OK");
	JButton			cancelButton = new JButton("Cancel");


	/**
	 * Constructor for relayEditT.
	 * 
	 * @param ttl
	 */
	public relayEditT(relayT relay) {
		super("Relay Setup");
		this.relay = relay;

		SimpleDateFormat _fmt = new SimpleDateFormat("HH:mm:ss");
		_fmt.setLenient(false);		
		DateFormatter fmt = new DateFormatter(_fmt);
		fmt.setAllowsInvalid(false);
		fmt.setOverwriteMode(true);		

		deviceT.loadRelaysComboBox(cbDev);
		
		cbType.addItem("clock");
		cbType.addItem("cycle");
		cbType.addItem("slave");
		cbType.addItem("cond");
		cbType.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				typeChanged();
			}
		});

		
		cbBoard.addItem("B2102");
		cbBoard.addItem("Parallel");

		for (int i=1; i<= 16; i++)
			cbChannel.addItem(""+i);


		fldName = new JTextField(10);
		fldTimeOn = new JFormattedTextField(fmt);
		fldTimeOn.setPreferredSize(new Dimension(70,20));		
		fldTimeOff = new JFormattedTextField(fmt);
		fldTimeOff.setPreferredSize(new Dimension(70,20));
		

		appPnl.setLayout(new BoxLayout(appPnl,BoxLayout.Y_AXIS));
		appTop.setLayout(new BoxLayout(appTop,BoxLayout.Y_AXIS));
		
		header1.setLayout(new BoxLayout(header1,BoxLayout.Y_AXIS));
		header2.setLayout(new BoxLayout(header2,BoxLayout.Y_AXIS));
		header3.setLayout(new BoxLayout(header3,BoxLayout.Y_AXIS));
		header(header1,new JLabel("Device Name:"),fldName);
		header(header1,new JLabel("Type:"),cbType);
		header(header1,new JLabel("Board:"),cbBoard,cbChannel);
		
		header(header2,lblTimeOn,fldTimeOn);
		header(header2,lblTimeOff,fldTimeOff);
		header(header3,lblSlaveOf,cbDev);
		
		appTop.add(header1);
		appTop.add(header2);
		appTop.add(header3);
		
		appTop.add(Box.createVerticalStrut(8));


		
		LinkedList onList = null;
		LinkedList offList = null;
		if (relay != null) {
			onList = relay.getOnConditions();
			offList = relay.getOffConditions();
		}
		onTab = new conditionPnlT(this,onList,
					"<html>When the conditions are TRUE, the<br>"+
					"relay will change from ON to OFF</html>");
		
		offTab = new conditionPnlT(this,offList,
					"<html>When the conditions are TRUE, the<br>"+
					"relay will change from OFF to ON</html>");
		offTab.setVisible(false);
		tabbedPane.add(onTab,"On->Off Conditions");

		appBottom.add(tabbedPane);		
		appBottom.setBorder(new EmptyBorder(4,4,4,4));

		appPnl.add(appTop);
		appPnl.add(appBottom);		
		m_contentPane.add(appPnl, BorderLayout.CENTER);
		
		
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				okButtonAction();
			}
		});

		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cancelButtonAction();
			}
		});
		
		JPanel pnl = new JPanel();
		pnl.setLayout(new FlowLayout(FlowLayout.RIGHT));
		pnl.add(okButton);
		pnl.add(cancelButton);
		m_contentPane.add(pnl, BorderLayout.SOUTH);

		load();
		getRootPane().setDefaultButton(okButton);

		xcenter();
		setVisible(true);			
	}
	

	/**
	 * init fields
	 */
	private void load() {
		if (relay == null) {
			Date now = new Date();
			
			fldTimeOn.setValue(now);
			fldTimeOff.setValue(now);
		}
		else {
			fldName.setText(relay.getName());

			cbType.setSelectedItem(relay.getRelayTypStr());
			cbBoard.setSelectedItem(relay.getBoardName());
			cbChannel.setSelectedItem(Integer.toString(relay.getChannel()));


			Calendar	now = Calendar.getInstance();
			now.clear();						
			if (relay.getOnTime()==null)
				fldTimeOn.setValue(now.getTime());
			else
				fldTimeOn.setValue(relay.getOnTime());

			//now.add(Calendar.HOUR,1);
			if (relay.getOffTime()==null)
				fldTimeOff.setValue(now.getTime());
			else
				fldTimeOff.setValue(relay.getOffTime());
				
			relayT slaveOf = relay.getSlaveDevice();
			if (slaveOf != null) {
				cbDev.setSelectedItem(slaveOf.getName());
			}
		}
		typeChanged();
	}

	
	/**
	 * handle a change of type
	 */	
	void typeChanged() {
		String typ = (String)cbType.getSelectedItem();
		
		if (typ.equals("slave")) {
			header2.setVisible(false);
			header3.setVisible(true);

			//tabbedPane.setVisible(false);
			tabbedPane.setVisible(true);
			if (offTab.isVisible())
				tabbedPane.remove(offTab);
			offTab.setVisible(false);

		}
		else if (typ.equals("cond")) {
			header2.setVisible(false);
			header3.setVisible(false);

			tabbedPane.setVisible(true);
			if (offTab.isVisible()==false)
				tabbedPane.add(offTab,"Off->On Conditions");
			offTab.setVisible(true);
		}
		else if (typ.equals("clock")) {
			header2.setVisible(true);
			header3.setVisible(false);
			
			tabbedPane.setVisible(true);
			if (offTab.isVisible()==false)
				tabbedPane.add(offTab,"Off->On Conditions");
			offTab.setVisible(true);
		}
		else if (typ.equals("cycle")) {
			header2.setVisible(true);
			header3.setVisible(false);

			tabbedPane.setVisible(true);
			if (offTab.isVisible())
				tabbedPane.remove(offTab);
			offTab.setVisible(false);
		}
		pack();
	}
		
		
	/**
	 * handle OK button selection
	 */
	void okButtonAction() {
		relayT r = new relayT(fldName.getText());
		
		r.setRelayType((String)cbType.getSelectedItem());

		if (cbBoard.getSelectedIndex() == 0)
			r.setBoard(srv.b2102);
		else
			r.setBoard(srv.pport);
		r.setChannel(cbChannel.getSelectedIndex()+1);
			
			
		switch (r.getRelayTyp()) {
			case relayT.CLOCK:
			case relayT.CYCLE:
				Calendar now = Calendar.getInstance();
				now.setTime( (Date)fldTimeOn.getValue() );
				r.setTime(deviceT.ON,now.get(Calendar.HOUR_OF_DAY), 
										now.get(Calendar.MINUTE), 
										now.get(Calendar.SECOND));

				now.setTime( (Date)fldTimeOff.getValue() );
				r.setTime(deviceT.OFF,now.get(Calendar.HOUR_OF_DAY), 
										now.get(Calendar.MINUTE), 
										now.get(Calendar.SECOND));
				r.setOnConditions(onTab.iterator());
				r.setOffConditions(offTab.iterator());
				break;
			
			case relayT.SLAVE:
				relayT slaveOf = (relayT)deviceT.fnd((String)cbDev.getSelectedItem());
				if (slaveOf != null) {
					r.setSlaveDevice(slaveOf);
				}
				r.setOnConditions(onTab.iterator());
				r.setOffConditions(offTab.iterator());
				break;
				
			case relayT.COND:
				r.setOnConditions(onTab.iterator());
				r.setOffConditions(offTab.iterator());
				break;
		}
			
		sysMsgT.postMsg(sysMsgT.MSG_EDIT_RELAY, r, relay);
		dispose();
	}


	/**
	 * handle CANCEL button selection
	 */
	void cancelButtonAction() {
		dispose();
	}

		
	/**
	 * 
	 * @param pnl
	 * @param lbl
	 * @param cb
	 */
	void header(JPanel pnl, JLabel lbl, JComponent cb) {
		lbl.setPreferredSize(new Dimension(90,20));
		lbl.setHorizontalAlignment(SwingConstants.RIGHT);
		lbl.setAlignmentX(Component.RIGHT_ALIGNMENT);


		JPanel	p = new JPanel();
		p.setLayout(new FlowLayout(FlowLayout.LEFT));
		p.add(lbl);
		p.add(cb);
		pnl.add(p);
	}


	/**
	 * 
	 * @param pnl
	 * @param lbl
	 * @param cb1
	 * @param cb2
	 */
	void header(JPanel pnl, JLabel lbl, JComponent cb1, JComponent cb2) {
		lbl.setPreferredSize(new Dimension(90,20));
		lbl.setHorizontalAlignment(SwingConstants.RIGHT);
		lbl.setAlignmentX(Component.RIGHT_ALIGNMENT);

		JPanel	p = new JPanel();
		p.setLayout(new FlowLayout(FlowLayout.LEFT));
		p.add(lbl);
		p.add(cb1);
		
		lbl = new JLabel("Channel:");
		lbl.setPreferredSize(new Dimension(90,20));
		lbl.setHorizontalAlignment(SwingConstants.RIGHT);
		lbl.setAlignmentX(Component.RIGHT_ALIGNMENT);
		p.add(lbl);
		p.add(cb2);
		
		pnl.add(p);
	}

}
