/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.ui;

import JAuto.srv.*;
import java.text.*;
import java.util.*;


public class relayTblModelT extends tableModelT {
	static final	int	NAME	= 0;
	static final	int	BOARD	= 1;
	static final	int 	CHANNEL	= 2;
	static final	int 	TYPE	= 3;
	static final	int 	STATE	= 4;
	static final	int	ONTIME	= 5;
	static final	int	OFFTIME	= 6;
	static final   int	COLUMN_CNT = OFFTIME+1;

	static	DateFormat		df1;
	
	relayTblT	tbl;
	String		onStr1 = "<HTML><FONT COLOR=GREEN>*On</HTML>";
	String		offStr1 ="<HTML><FONT COLOR=RED>*Off</HTML>";	
	String		onStr2 = "<HTML><FONT COLOR=GREEN>On</HTML>";
	String		offStr2 ="<HTML><FONT COLOR=RED>Off</HTML>";

	String	headers[] = {	"Name",
							"Board",
							"Channel",
							"Type",
							"State",
							"On time",
							"Off time"
						};
	
	/**
	 * Constructor for relayTblModelT.
	 */
	public relayTblModelT() {
		super();
		df1 = DateFormat.getTimeInstance(DateFormat.SHORT);
	}

	public int getColumnCount()                 { return COLUMN_CNT; }
	public String getColumnName(int col)        { return headers[col]; }
	
	
	/**
	 * 
	 */
	public Object getValueAt(int row, int col) {
		relayT	relay = (relayT)rowData.elementAt(row);
		
		switch (col) {
			case NAME:		return relay.getName();
			case BOARD:	return relay.getBoardName();
			case CHANNEL:	return Integer.toString(relay.getChannel());
			case TYPE:		return relay.getRelayTypStr();
			case STATE:	if (relay.isOverRide())	
								return (relay.getState()==true) ? onStr1:offStr1;
							return (relay.getState()==true) ? onStr2:offStr2;
			
			case ONTIME:	
				if (relay.getRelayTyp() == relayT.CLOCK) 
					return (relay.getOnTime()==null) ? "":df1.format(relay.getOnTime());
				if (relay.getRelayTyp() == relayT.SLAVE) {
					if (relay.getSlaveDevice() == null)
						return "";
					else
						return relay.getSlaveDevice().getName();
				}
				if (relay.getRelayTyp() == relayT.COND)
					return "";
				return relay.getOnCalendar().get(Calendar.HOUR_OF_DAY) + ":" +
						relay.getOnCalendar().get(Calendar.MINUTE) + ":" +
						relay.getOnCalendar().get(Calendar.SECOND);

			case OFFTIME:
				if (relay.getRelayTyp() == relayT.CLOCK) 
					return (relay.getOffTime()==null) ? "":df1.format(relay.getOffTime());
				if (relay.getRelayTyp()==relayT.SLAVE || relay.getRelayTyp()==relayT.COND)
					return "";
				return relay.getOffCalendar().get(Calendar.HOUR_OF_DAY) + ":" +
						relay.getOffCalendar().get(Calendar.MINUTE) + ":" +
						relay.getOffCalendar().get(Calendar.SECOND);
		}
		return "?";
	}
	

	/**
	 * update a relay in the table model
	 * 
	 * @param relay
	 */
	public void upd(relayT relay) {
		if (relay == null)
			return;
		
		if (fnd(relay.getName()) != null) {
			fireUpd(relay.getTblRow());
		}
		else {
			rowData.add(relay);
			fireInserted(getRowCount()-1);
			calcRowNumers();
		}
	}
	

	/**
	 * delete a relay from the table model
	 * 
	 * @param name
	 */
	public void delete(String name) {
		relayT r = fnd(name);
		
		if (r != null) {
			fireDeleted(r.getTblRow());
			rowData.remove(r);
			calcRowNumers();
		}
	}
	
	
	/**
	 * find a relay in the table model
	 * 
	 * @param name
	 * 
	 * @return relayT
	 */
	relayT fnd(String name) {
		Iterator itr = iterator();
		
		while (itr.hasNext()) {
			relayT relay = (relayT)itr.next();
			if (name.equals(relay.getName()))
				return relay;			
		}
		
		return null;
	}
	
	/**
	 * recalc row numbers for table display
	 */
	void calcRowNumers() {
		int		row=0;
		Iterator 	itr = iterator();

		while (itr.hasNext()) {
			relayT relay = (relayT)itr.next();
			relay.setTblRow(row++);
		}
	}

}
