/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.ui;

import JAuto.srv.*;
import java.util.*;

public class sensorTblModelT extends tableModelT {
	static final	int	NAME	= 0;
	static final	int	BOARD	= 1;
	static final	int 	CHANNEL	= 2;
	static final	int	VALUE	= 3;
	static final	int	AVG		= 4;
	static final   int	COLUMN_CNT = AVG+1;
	
	String	headers[] = {	"Name",
							"Board",
							"Channel",
							"Value",
							"SQL Avg"
						};
	
	/**
	 * Constructor for sensorTblModelT.
	 */
	public sensorTblModelT() {
		super();
	}

	public int getColumnCount()                 { return COLUMN_CNT; }
	public String getColumnName(int col)        { return headers[col]; }
	
	/**
	 * 
	 */
	public Object getValueAt(int row, int col) {
		sensorT	sensor = (sensorT)rowData.elementAt(row);
		
		switch (col) {
			case NAME:		return sensor.getName();
			case BOARD:	return sensor.getBoardName();
			case CHANNEL:	return Integer.toString(sensor.getChannel());
			case VALUE:	return ""+sensor.getSensorDat();
			case AVG:		return ""+sensor.getAvg();
		}
		return "?";
	}
	
	
	/**
	 * update a sensor in the table model
	 * 
	 * @param sensor
	 */
	public void upd(sensorT sensor) {
		if (sensor == null)
			return;
		
		if (fnd(sensor.getName()) != null) {
			fireUpd(sensor.getTblRow());
		}
		else {
			rowData.add(sensor);
			fireInserted(getRowCount()-1);
			calcRowNumers();
		}
	}
	

	/**
	 * delete a sensor from the table model
	 * 
	 * @param name
	 */
	public void delete(String name) {
		sensorT s = fnd(name);
		
		if (s != null) {
			rowData.remove(s);
			fireDeleted(s.getTblRow());
			calcRowNumers();
		}
	}
	
	
	/**
	 * find a sensor in the table model
	 * 
	 * @param name
	 * 
	 * @return sensorT
	 */
	sensorT fnd(String name) {
		Iterator itr = iterator();
		
		while (itr.hasNext()) {
			sensorT sensor = (sensorT)itr.next();
			if (name.equals(sensor.getName()))
				return sensor;			
		}		
		return null;
	}
	
	
	/**
	 * recalc row numbers for table display
	 */
	void calcRowNumers() {
		int		row=0;
		Iterator 	itr = iterator();

		while (itr.hasNext()) {
			sensorT sensor = (sensorT)itr.next();
			sensor.setTblRow(row++);
		}
	}
}
