/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package JAuto.util;

import java.util.*;

/**
 * System message class
 */
public class sysMsgT {
	public static final int	MSG_LOGON		= 0;
	public static final int	MSG_LOGOFF		= 1;
	public static final int	MSG_CONNECTION	= 2;
	public static final int	MSG_OVERRIDE	= 3;
	public static final int	MSG_PROC_RELAY	= 4;
	public static final int	MSG_PROC_SENSOR	= 5;
	public static final int	MSG_EDIT_RELAY	= 6;
	public static final int	MSG_EDIT_SENSOR	= 7;
	public static final int	MSG_EDIT_BOARD	= 8;
	public static final int	MSG_DEL_RELAY	= 9;
	public static final int	MSG_DEL_SENSOR	= 10;
	public static final int	MSG_CLIENT		= 11;
	public static final int	MSG_CLR_SENSOR	= 12;
	

	
    /** system msg queue */
    private static LinkedList    _queue = new LinkedList();

    /** pre-allocated system msg's */
    private static LinkedList    _msgs = new LinkedList();


    /** command code */
    public int     cmd;

    /** command int parm1 */
    public int     int1;

    /** command int parm2 */
    public int     int2;

    /** command object parm1 */
    public Object  obj1;

    /** command object parm2 */
    public Object  obj2;


    /**
     * constructor
     *
     * @param   cmd command code
     */
    public sysMsgT() {
    }


    /**
     * return a message to the system queue
     *
     * @param  the message to return
     */
    private static sysMsgT getMsg() {
        sysMsgT msg;

        synchronized(_msgs) {
            if (_msgs.size() == 0) {
                msg = new sysMsgT();
            }
            else {
                msg = (sysMsgT)_msgs.removeFirst();
            }
        }
        return msg;
    }

    /**
     * return a message to the system queue
     *
     * @param  the message to return
     */
    public static void releaseMsg(sysMsgT msg) {
        synchronized(_msgs) {
            _msgs.add(msg);
        }
    }

    /**
     * post a message to the system queue
     *
     * @param  cmd      message cmd to post
     */
    public static void postMsg(int cmd) {
        sysMsgT msg = getMsg();

        msg.cmd = cmd;
        synchronized(_queue) {
            _queue.add(msg);
            _queue.notify();
        }
    }


	/**
	 * post a message to the system queue
	 *
	 * @param  cmd      message cmd to post
	 * @param  obj1     user object 1
     * @param  int1     user int 1
     * @param  int2     user int 2
	 */
	public static void postMsg(int cmd, Object obj1, int int1, int int2) {
		sysMsgT msg = getMsg();

		msg.cmd = cmd;
		msg.obj1 = obj1;
		msg.int1 = int1;
		msg.int2 = int2;
		
		synchronized(_queue) {
			_queue.add(msg);
			_queue.notify();
		}
	}


    /**
     * post a message to the system queue
     *
     * @param  cmd      message cmd to post
     * @param  obj1     user object 1
     * @param  obj2     user object 2
     */
    public static void postMsg(int cmd, Object obj1, Object obj2) {
        sysMsgT msg = getMsg();

        msg.cmd  = cmd;
        msg.obj1 = obj1;
        msg.obj2 = obj2;
        synchronized(_queue) {
            _queue.add(msg);
            _queue.notify();
        }
    }

	/**
	 * post a message to the system queue
	 *
	 * @param  cmd      message cmd to post
	 * @param  obj1     user object 1
	 */
	public static void postMsg(int cmd, Object obj1) {
		sysMsgT msg = getMsg();

		msg.cmd  = cmd;
		msg.obj1 = obj1;
		synchronized(_queue) {
			_queue.add(msg);
			_queue.notify();
		}
	}

	/**
	 * post a message to the system queue
	 *
	 * @param  cmd      message cmd to post
	 * @param  obj1     user object 1
	 * @param  int1     user int 1
	 */
	public static void postMsg(int cmd, Object obj1, int int1) {
		sysMsgT msg = getMsg();

		msg.cmd  = cmd;
		msg.obj1 = obj1;
		msg.int1 = int1;
		synchronized(_queue) {
			_queue.add(msg);
			_queue.notify();
		}
	}



    /**
     * read a message from the system queue
     *
     * @param  the message to store result into
     */
    public static sysMsgT readMsg() {
        synchronized(_queue) {
            while (_queue.size() == 0) {
                try { _queue.wait(); }
                catch (InterruptedException e) {
                    // forced interruption; quit thread.
                    return null;
                }
            }
            sysMsgT msg = (sysMsgT)_queue.removeFirst();
            return msg;
        }
    }



    /**
     * dump contents to screen
     */
    public void dump() {
        System.out.println("cmd="+cmd+
                           " int1="+int1+
                           " int2="+int2+
                           " obj1="+obj1+
                           " obj2="+obj2);
    }


}

