/**
 *  Copyright (c) 2002-2003, ngc7579
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  NOTE: build with "gcc -O2 -Wall -fpic -shared ioPortT.c -o libioPortT.so"
 */
#include "ioPortT.h"
#include <unistd.h>
#include <stdio.h>
#include <sys/io.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>

static  int m_fd  = -1;
static  int _read(jint addr);
static  int _write(jbyte dat, jint addr);
static  int _seek(jint addr);
static void _delay(void);

/*
 * Class:     JAuto_srv_ioPortT
 * Method:    ioOpen
 * Signature: (III)I
 */
JNIEXPORT jint JNICALL Java_JAuto_srv_ioPortT_ioOpen
  (JNIEnv *env, jobject obj)
{
    if (m_fd == -1) {
        m_fd = open("/dev/port", O_RDWR);
        if (m_fd < 0)
            return -1;
    }

    return 0;
}


/*
 * Class:     JAuto_srv_ioPortT
 * Method:    ioClose
 * Signature: (III)I
 */
JNIEXPORT jint JNICALL Java_JAuto_srv_ioPortT_ioClose
  (JNIEnv *env, jobject obj)
{
    if (m_fd != -1) {
        close(m_fd);
        m_fd = -1;
    }
    return 0;
}


/*
 * Class:     ioPortT
 * Method:    readPort
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_JAuto_srv_ioPortT_ioReadPort__I
  (JNIEnv *env, jobject obj, jint addr)
{
    return _read(addr);
}


/*
 * Class:     ioPortT
 * Method:    readPort
 * Signature: (IJ)I
 */
JNIEXPORT jint JNICALL Java_JAuto_srv_ioPortT_ioReadPort__IJ
  (JNIEnv *env, jobject obj, jint addr, jlong usec)
{
    if (usec > 0)
        usleep(usec);
    return _read(addr);
}


/*
 * Class:     ioPortT
 * Method:
 * Signature: (II)I
 */
JNIEXPORT jint JNICALL Java_JAuto_srv_ioPortT_ioReadB3001__II
  (JNIEnv *env, jobject obj, jint addr, jint channel0)
{
    int   data=0;
    int   selch   = channel0&0x0F;   // mask high half bits
    int   pulseh  = selch|0x40;		// set trigger bit high
    int   pulsel  = selch&0x0F;		// set trigger bit low
    int   lbe     = selch|0x20;		// low byte data enable
    int   hbe     = selch|0x10;		// high byte data enable
    int   datal,datah;		        // low & high byte of data
    int   selp    = 1+addr;	            // channel selection port offset
    int   ctrlp   = 1+addr;	            // A/D control port offset
    int   statusp = 0+addr;		        // A/D status port offset
    int   datap   = 1+addr;	            // A/D data port offset
    clock_t t;

    //
    // select channel for conversion
    //
    _write(selch,selp);
    _delay();

    _write(pulseh,ctrlp);
    _write(pulsel,ctrlp);
    _delay();

    t = clock() + CLOCKS_PER_SEC/4;         //wait up to 1/4 seconds
    while ((_read(statusp)&0x40)==0x40) {
        //
        // conversion timeout?
        //
        if (clock() >= t) {
            printf("B3001 read channel:%d addr=%04X TIMEOUT\n", channel0,addr);
            return -1;
        }
    }

    _write(lbe,ctrlp);	            // select low byte of data
    datal = _read(datap);	        // read low byte of data

    _write(hbe,ctrlp);	            // select high byte of data
    datah = _read(datap);           // read high byte of data

    if (datah >= 0x80)			            // is negative data?
        data = ((datah&0x0F)*256+datal);	// yes,it's POSITIVE data
    else
  	    data =- ((datah&0x0F)*256+datal);   // it's NEGATIVE data

    return (data * 5000) / 4096;
}


/*
 * Class:     ioPortT
 * Method:    writePort
 * Signature: (IB)I
 */
JNIEXPORT jint JNICALL Java_JAuto_srv_ioPortT_ioWritePort__IB
  (JNIEnv *env, jobject obj, jint addr, jbyte b)
{
    return _write(b,addr);
}


/*
 * Class:     ioPortT
 * Method:    writePort
 * Signature: (IBJ)I
 */
JNIEXPORT jint JNICALL Java_JAuto_srv_ioPortT_ioWritePort__IBJ
  (JNIEnv *env, jobject obj, jint addr, jbyte b, jlong usec)
{
    if (usec > 0)
        usleep(usec);
    return Java_JAuto_srv_ioPortT_ioWritePort__IB(env, obj, addr, b);
}


static int _read(jint addr)
{
    int           r;
    unsigned char c;

    if (_seek(addr) != 0) {
        printf("_read seek error addr=0x%04X\n",addr);
        return -2;
    }

    _delay();
    r = read(m_fd,&c,1);
    if (r != 1) {
        printf("_read error addr=0x%04X\n",addr);
        return -3;
    }
    return c;
}

static int _write(jbyte b, jint addr)
{
    if (_seek(addr) != 0) {
        printf("writePort seek error addr=0x%04X byte=%02X\n",addr,b);
        return -2;
    }

    _delay();
    int r = write(m_fd,&b,1);
    if (r != 1) {
        printf("writePort write error addr=0x%04X byte=%02X\n",addr,b);
        return -3;
    }
    return 0;
}

static int _seek(jint addr)
{
    if (m_fd == -1)
        return -1;

    long seekResult = lseek(m_fd,addr,0);
    if (seekResult < 0) {
        return -2;
    }

    if ((int)seekResult != addr) {
        return -3;
    }
    return 0;
}

static void _delay(void)
{
    usleep(100);
}
